# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/qemu/ui/qemu_vm_configuration_page.ui'
#
# Created by: PyQt5 UI code generator 5.5.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_QemuVMConfigPageWidget(object):
    def setupUi(self, QemuVMConfigPageWidget):
        QemuVMConfigPageWidget.setObjectName("QemuVMConfigPageWidget")
        QemuVMConfigPageWidget.resize(655, 696)
        self.verticalLayout = QtWidgets.QVBoxLayout(QemuVMConfigPageWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiQemutabWidget = QtWidgets.QTabWidget(QemuVMConfigPageWidget)
        self.uiQemutabWidget.setObjectName("uiQemutabWidget")
        self.uiGeneralSettingsTab = QtWidgets.QWidget()
        self.uiGeneralSettingsTab.setObjectName("uiGeneralSettingsTab")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.uiGeneralSettingsTab)
        self.gridLayout_4.setContentsMargins(10, 10, 10, 10)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.uiRamLabel = QtWidgets.QLabel(self.uiGeneralSettingsTab)
        self.uiRamLabel.setObjectName("uiRamLabel")
        self.gridLayout_4.addWidget(self.uiRamLabel, 4, 0, 1, 1)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.uiSymbolLineEdit = QtWidgets.QLineEdit(self.uiGeneralSettingsTab)
        self.uiSymbolLineEdit.setObjectName("uiSymbolLineEdit")
        self.horizontalLayout_7.addWidget(self.uiSymbolLineEdit)
        self.uiSymbolToolButton = QtWidgets.QToolButton(self.uiGeneralSettingsTab)
        self.uiSymbolToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiSymbolToolButton.setObjectName("uiSymbolToolButton")
        self.horizontalLayout_7.addWidget(self.uiSymbolToolButton)
        self.gridLayout_4.addLayout(self.horizontalLayout_7, 2, 1, 1, 1)
        self.uiCategoryComboBox = QtWidgets.QComboBox(self.uiGeneralSettingsTab)
        self.uiCategoryComboBox.setObjectName("uiCategoryComboBox")
        self.gridLayout_4.addWidget(self.uiCategoryComboBox, 3, 1, 1, 1)
        self.uiRamSpinBox = QtWidgets.QSpinBox(self.uiGeneralSettingsTab)
        self.uiRamSpinBox.setMinimum(32)
        self.uiRamSpinBox.setMaximum(65535)
        self.uiRamSpinBox.setProperty("value", 256)
        self.uiRamSpinBox.setObjectName("uiRamSpinBox")
        self.gridLayout_4.addWidget(self.uiRamSpinBox, 4, 1, 1, 1)
        self.uiSymbolLabel = QtWidgets.QLabel(self.uiGeneralSettingsTab)
        self.uiSymbolLabel.setObjectName("uiSymbolLabel")
        self.gridLayout_4.addWidget(self.uiSymbolLabel, 2, 0, 1, 1)
        self.uiNameLineEdit = QtWidgets.QLineEdit(self.uiGeneralSettingsTab)
        self.uiNameLineEdit.setObjectName("uiNameLineEdit")
        self.gridLayout_4.addWidget(self.uiNameLineEdit, 0, 1, 1, 1)
        self.uiConsoleTypeComboBox = QtWidgets.QComboBox(self.uiGeneralSettingsTab)
        self.uiConsoleTypeComboBox.setObjectName("uiConsoleTypeComboBox")
        self.uiConsoleTypeComboBox.addItem("")
        self.uiConsoleTypeComboBox.addItem("")
        self.uiConsoleTypeComboBox.addItem("")
        self.gridLayout_4.addWidget(self.uiConsoleTypeComboBox, 8, 1, 1, 1)
        self.uiConsoleTypeLabel = QtWidgets.QLabel(self.uiGeneralSettingsTab)
        self.uiConsoleTypeLabel.setObjectName("uiConsoleTypeLabel")
        self.gridLayout_4.addWidget(self.uiConsoleTypeLabel, 8, 0, 1, 1)
        self.uiBootPriorityLabel = QtWidgets.QLabel(self.uiGeneralSettingsTab)
        self.uiBootPriorityLabel.setObjectName("uiBootPriorityLabel")
        self.gridLayout_4.addWidget(self.uiBootPriorityLabel, 7, 0, 1, 1)
        self.uiQemuListLabel = QtWidgets.QLabel(self.uiGeneralSettingsTab)
        self.uiQemuListLabel.setObjectName("uiQemuListLabel")
        self.gridLayout_4.addWidget(self.uiQemuListLabel, 6, 0, 1, 1)
        self.uiCategoryLabel = QtWidgets.QLabel(self.uiGeneralSettingsTab)
        self.uiCategoryLabel.setObjectName("uiCategoryLabel")
        self.gridLayout_4.addWidget(self.uiCategoryLabel, 3, 0, 1, 1)
        self.uiCPUSpinBox = QtWidgets.QSpinBox(self.uiGeneralSettingsTab)
        self.uiCPUSpinBox.setMinimum(1)
        self.uiCPUSpinBox.setMaximum(255)
        self.uiCPUSpinBox.setObjectName("uiCPUSpinBox")
        self.gridLayout_4.addWidget(self.uiCPUSpinBox, 5, 1, 1, 1)
        self.uiBootPriorityComboBox = QtWidgets.QComboBox(self.uiGeneralSettingsTab)
        self.uiBootPriorityComboBox.setObjectName("uiBootPriorityComboBox")
        self.gridLayout_4.addWidget(self.uiBootPriorityComboBox, 7, 1, 1, 1)
        self.uiNameLabel = QtWidgets.QLabel(self.uiGeneralSettingsTab)
        self.uiNameLabel.setObjectName("uiNameLabel")
        self.gridLayout_4.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(263, 94, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem, 9, 1, 1, 1)
        self.uiQemuListComboBox = QtWidgets.QComboBox(self.uiGeneralSettingsTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiQemuListComboBox.sizePolicy().hasHeightForWidth())
        self.uiQemuListComboBox.setSizePolicy(sizePolicy)
        self.uiQemuListComboBox.setObjectName("uiQemuListComboBox")
        self.gridLayout_4.addWidget(self.uiQemuListComboBox, 6, 1, 1, 1)
        self.uiCPULabel = QtWidgets.QLabel(self.uiGeneralSettingsTab)
        self.uiCPULabel.setObjectName("uiCPULabel")
        self.gridLayout_4.addWidget(self.uiCPULabel, 5, 0, 1, 1)
        self.uiDefaultNameFormatLabel = QtWidgets.QLabel(self.uiGeneralSettingsTab)
        self.uiDefaultNameFormatLabel.setObjectName("uiDefaultNameFormatLabel")
        self.gridLayout_4.addWidget(self.uiDefaultNameFormatLabel, 1, 0, 1, 1)
        self.uiDefaultNameFormatLineEdit = QtWidgets.QLineEdit(self.uiGeneralSettingsTab)
        self.uiDefaultNameFormatLineEdit.setObjectName("uiDefaultNameFormatLineEdit")
        self.gridLayout_4.addWidget(self.uiDefaultNameFormatLineEdit, 1, 1, 1, 1)
        self.uiQemutabWidget.addTab(self.uiGeneralSettingsTab, "")
        self.uiHddTab = QtWidgets.QWidget()
        self.uiHddTab.setObjectName("uiHddTab")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.uiHddTab)
        self.verticalLayout_3.setContentsMargins(10, 10, 10, 10)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.uiHdaGroupBox = QtWidgets.QGroupBox(self.uiHddTab)
        self.uiHdaGroupBox.setObjectName("uiHdaGroupBox")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.uiHdaGroupBox)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.uiHdaDiskImageLabel = QtWidgets.QLabel(self.uiHdaGroupBox)
        self.uiHdaDiskImageLabel.setObjectName("uiHdaDiskImageLabel")
        self.gridLayout_6.addWidget(self.uiHdaDiskImageLabel, 0, 0, 1, 1)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.uiHdaDiskImageLineEdit = QtWidgets.QLineEdit(self.uiHdaGroupBox)
        self.uiHdaDiskImageLineEdit.setObjectName("uiHdaDiskImageLineEdit")
        self.horizontalLayout_8.addWidget(self.uiHdaDiskImageLineEdit)
        self.uiHdaDiskImageToolButton = QtWidgets.QToolButton(self.uiHdaGroupBox)
        self.uiHdaDiskImageToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiHdaDiskImageToolButton.setObjectName("uiHdaDiskImageToolButton")
        self.horizontalLayout_8.addWidget(self.uiHdaDiskImageToolButton)
        self.uiHdaDiskImageCreateToolButton = QtWidgets.QToolButton(self.uiHdaGroupBox)
        self.uiHdaDiskImageCreateToolButton.setObjectName("uiHdaDiskImageCreateToolButton")
        self.horizontalLayout_8.addWidget(self.uiHdaDiskImageCreateToolButton)
        self.gridLayout_6.addLayout(self.horizontalLayout_8, 0, 1, 1, 1)
        self.uiHdaDiskInterfaceLabel = QtWidgets.QLabel(self.uiHdaGroupBox)
        self.uiHdaDiskInterfaceLabel.setObjectName("uiHdaDiskInterfaceLabel")
        self.gridLayout_6.addWidget(self.uiHdaDiskInterfaceLabel, 1, 0, 1, 1)
        self.uiHdaDiskInterfaceComboBox = QtWidgets.QComboBox(self.uiHdaGroupBox)
        self.uiHdaDiskInterfaceComboBox.setObjectName("uiHdaDiskInterfaceComboBox")
        self.gridLayout_6.addWidget(self.uiHdaDiskInterfaceComboBox, 1, 1, 1, 1)
        self.verticalLayout_3.addWidget(self.uiHdaGroupBox)
        self.uiHdbgroupBox = QtWidgets.QGroupBox(self.uiHddTab)
        self.uiHdbgroupBox.setObjectName("uiHdbgroupBox")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.uiHdbgroupBox)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.uiHdbDiskImageLabel = QtWidgets.QLabel(self.uiHdbgroupBox)
        self.uiHdbDiskImageLabel.setObjectName("uiHdbDiskImageLabel")
        self.gridLayout_7.addWidget(self.uiHdbDiskImageLabel, 0, 0, 1, 1)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.uiHdbDiskImageLineEdit = QtWidgets.QLineEdit(self.uiHdbgroupBox)
        self.uiHdbDiskImageLineEdit.setObjectName("uiHdbDiskImageLineEdit")
        self.horizontalLayout_4.addWidget(self.uiHdbDiskImageLineEdit)
        self.uiHdbDiskImageToolButton = QtWidgets.QToolButton(self.uiHdbgroupBox)
        self.uiHdbDiskImageToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiHdbDiskImageToolButton.setObjectName("uiHdbDiskImageToolButton")
        self.horizontalLayout_4.addWidget(self.uiHdbDiskImageToolButton)
        self.uiHdbDiskImageCreateToolButton = QtWidgets.QToolButton(self.uiHdbgroupBox)
        self.uiHdbDiskImageCreateToolButton.setObjectName("uiHdbDiskImageCreateToolButton")
        self.horizontalLayout_4.addWidget(self.uiHdbDiskImageCreateToolButton)
        self.gridLayout_7.addLayout(self.horizontalLayout_4, 0, 1, 1, 1)
        self.uiHdbDiskInterfaceLabel = QtWidgets.QLabel(self.uiHdbgroupBox)
        self.uiHdbDiskInterfaceLabel.setObjectName("uiHdbDiskInterfaceLabel")
        self.gridLayout_7.addWidget(self.uiHdbDiskInterfaceLabel, 1, 0, 1, 1)
        self.uiHdbDiskInterfaceComboBox = QtWidgets.QComboBox(self.uiHdbgroupBox)
        self.uiHdbDiskInterfaceComboBox.setObjectName("uiHdbDiskInterfaceComboBox")
        self.gridLayout_7.addWidget(self.uiHdbDiskInterfaceComboBox, 1, 1, 1, 1)
        self.verticalLayout_3.addWidget(self.uiHdbgroupBox)
        self.uiHdcGroupBox = QtWidgets.QGroupBox(self.uiHddTab)
        self.uiHdcGroupBox.setObjectName("uiHdcGroupBox")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.uiHdcGroupBox)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.uiHdcDiskImageLabel = QtWidgets.QLabel(self.uiHdcGroupBox)
        self.uiHdcDiskImageLabel.setObjectName("uiHdcDiskImageLabel")
        self.gridLayout_8.addWidget(self.uiHdcDiskImageLabel, 0, 0, 1, 1)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.uiHdcDiskImageLineEdit = QtWidgets.QLineEdit(self.uiHdcGroupBox)
        self.uiHdcDiskImageLineEdit.setObjectName("uiHdcDiskImageLineEdit")
        self.horizontalLayout_9.addWidget(self.uiHdcDiskImageLineEdit)
        self.uiHdcDiskImageToolButton = QtWidgets.QToolButton(self.uiHdcGroupBox)
        self.uiHdcDiskImageToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiHdcDiskImageToolButton.setObjectName("uiHdcDiskImageToolButton")
        self.horizontalLayout_9.addWidget(self.uiHdcDiskImageToolButton)
        self.uiHdcDiskImageCreateToolButton = QtWidgets.QToolButton(self.uiHdcGroupBox)
        self.uiHdcDiskImageCreateToolButton.setObjectName("uiHdcDiskImageCreateToolButton")
        self.horizontalLayout_9.addWidget(self.uiHdcDiskImageCreateToolButton)
        self.gridLayout_8.addLayout(self.horizontalLayout_9, 0, 1, 1, 1)
        self.uiHdcDiskInterfaceLabel = QtWidgets.QLabel(self.uiHdcGroupBox)
        self.uiHdcDiskInterfaceLabel.setObjectName("uiHdcDiskInterfaceLabel")
        self.gridLayout_8.addWidget(self.uiHdcDiskInterfaceLabel, 1, 0, 1, 1)
        self.uiHdcDiskInterfaceComboBox = QtWidgets.QComboBox(self.uiHdcGroupBox)
        self.uiHdcDiskInterfaceComboBox.setObjectName("uiHdcDiskInterfaceComboBox")
        self.gridLayout_8.addWidget(self.uiHdcDiskInterfaceComboBox, 1, 1, 1, 1)
        self.verticalLayout_3.addWidget(self.uiHdcGroupBox)
        self.uiHddGroupBox = QtWidgets.QGroupBox(self.uiHddTab)
        self.uiHddGroupBox.setObjectName("uiHddGroupBox")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.uiHddGroupBox)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.uiHddDiskImageLabel = QtWidgets.QLabel(self.uiHddGroupBox)
        self.uiHddDiskImageLabel.setObjectName("uiHddDiskImageLabel")
        self.gridLayout_9.addWidget(self.uiHddDiskImageLabel, 0, 0, 1, 1)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.uiHddDiskImageLineEdit = QtWidgets.QLineEdit(self.uiHddGroupBox)
        self.uiHddDiskImageLineEdit.setObjectName("uiHddDiskImageLineEdit")
        self.horizontalLayout_10.addWidget(self.uiHddDiskImageLineEdit)
        self.uiHddDiskImageToolButton = QtWidgets.QToolButton(self.uiHddGroupBox)
        self.uiHddDiskImageToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiHddDiskImageToolButton.setObjectName("uiHddDiskImageToolButton")
        self.horizontalLayout_10.addWidget(self.uiHddDiskImageToolButton)
        self.uiHddDiskImageCreateToolButton = QtWidgets.QToolButton(self.uiHddGroupBox)
        self.uiHddDiskImageCreateToolButton.setObjectName("uiHddDiskImageCreateToolButton")
        self.horizontalLayout_10.addWidget(self.uiHddDiskImageCreateToolButton)
        self.gridLayout_9.addLayout(self.horizontalLayout_10, 0, 1, 2, 1)
        self.uiHddDiskInterfaceLabel = QtWidgets.QLabel(self.uiHddGroupBox)
        self.uiHddDiskInterfaceLabel.setObjectName("uiHddDiskInterfaceLabel")
        self.gridLayout_9.addWidget(self.uiHddDiskInterfaceLabel, 1, 0, 2, 1)
        self.uiHddDiskInterfaceComboBox = QtWidgets.QComboBox(self.uiHddGroupBox)
        self.uiHddDiskInterfaceComboBox.setObjectName("uiHddDiskInterfaceComboBox")
        self.gridLayout_9.addWidget(self.uiHddDiskInterfaceComboBox, 2, 1, 1, 1)
        self.verticalLayout_3.addWidget(self.uiHddGroupBox)
        spacerItem1 = QtWidgets.QSpacerItem(438, 257, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem1)
        self.uiQemutabWidget.addTab(self.uiHddTab, "")
        self.uiCdromTab = QtWidgets.QWidget()
        self.uiCdromTab.setObjectName("uiCdromTab")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.uiCdromTab)
        self.verticalLayout_4.setContentsMargins(10, 10, 10, 10)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.uiCdromGroupBox = QtWidgets.QGroupBox(self.uiCdromTab)
        self.uiCdromGroupBox.setObjectName("uiCdromGroupBox")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.uiCdromGroupBox)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.uiCdromImageLabel = QtWidgets.QLabel(self.uiCdromGroupBox)
        self.uiCdromImageLabel.setObjectName("uiCdromImageLabel")
        self.gridLayout_10.addWidget(self.uiCdromImageLabel, 0, 0, 1, 1)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.uiCdromImageLineEdit = QtWidgets.QLineEdit(self.uiCdromGroupBox)
        self.uiCdromImageLineEdit.setObjectName("uiCdromImageLineEdit")
        self.horizontalLayout_11.addWidget(self.uiCdromImageLineEdit)
        self.uiCdromImageToolButton = QtWidgets.QToolButton(self.uiCdromGroupBox)
        self.uiCdromImageToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiCdromImageToolButton.setObjectName("uiCdromImageToolButton")
        self.horizontalLayout_11.addWidget(self.uiCdromImageToolButton)
        self.gridLayout_10.addLayout(self.horizontalLayout_11, 0, 1, 1, 1)
        self.verticalLayout_4.addWidget(self.uiCdromGroupBox)
        spacerItem2 = QtWidgets.QSpacerItem(20, 381, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem2)
        self.uiQemutabWidget.addTab(self.uiCdromTab, "")
        self.uiNetworkTab = QtWidgets.QWidget()
        self.uiNetworkTab.setObjectName("uiNetworkTab")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.uiNetworkTab)
        self.gridLayout_5.setContentsMargins(10, 10, 10, 10)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.uiAdapterTypesComboBox = QtWidgets.QComboBox(self.uiNetworkTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiAdapterTypesComboBox.sizePolicy().hasHeightForWidth())
        self.uiAdapterTypesComboBox.setSizePolicy(sizePolicy)
        self.uiAdapterTypesComboBox.setObjectName("uiAdapterTypesComboBox")
        self.gridLayout_5.addWidget(self.uiAdapterTypesComboBox, 6, 1, 1, 1)
        self.uiLegacyNetworkingCheckBox = QtWidgets.QCheckBox(self.uiNetworkTab)
        self.uiLegacyNetworkingCheckBox.setObjectName("uiLegacyNetworkingCheckBox")
        self.gridLayout_5.addWidget(self.uiLegacyNetworkingCheckBox, 7, 0, 1, 2)
        self.uiMacAddrLineEdit = QtWidgets.QLineEdit(self.uiNetworkTab)
        self.uiMacAddrLineEdit.setObjectName("uiMacAddrLineEdit")
        self.gridLayout_5.addWidget(self.uiMacAddrLineEdit, 5, 1, 1, 1)
        self.uiPortSegmentSizeLabel = QtWidgets.QLabel(self.uiNetworkTab)
        self.uiPortSegmentSizeLabel.setObjectName("uiPortSegmentSizeLabel")
        self.gridLayout_5.addWidget(self.uiPortSegmentSizeLabel, 3, 0, 1, 1)
        self.uiAdapterTypesLabel = QtWidgets.QLabel(self.uiNetworkTab)
        self.uiAdapterTypesLabel.setObjectName("uiAdapterTypesLabel")
        self.gridLayout_5.addWidget(self.uiAdapterTypesLabel, 6, 0, 1, 1)
        self.uiPortSegmentSizeSpinBox = QtWidgets.QSpinBox(self.uiNetworkTab)
        self.uiPortSegmentSizeSpinBox.setMaximum(128)
        self.uiPortSegmentSizeSpinBox.setSingleStep(4)
        self.uiPortSegmentSizeSpinBox.setObjectName("uiPortSegmentSizeSpinBox")
        self.gridLayout_5.addWidget(self.uiPortSegmentSizeSpinBox, 3, 1, 1, 1)
        self.uiAdaptersLabel = QtWidgets.QLabel(self.uiNetworkTab)
        self.uiAdaptersLabel.setObjectName("uiAdaptersLabel")
        self.gridLayout_5.addWidget(self.uiAdaptersLabel, 0, 0, 1, 1)
        self.uiMacAddrLabel = QtWidgets.QLabel(self.uiNetworkTab)
        self.uiMacAddrLabel.setObjectName("uiMacAddrLabel")
        self.gridLayout_5.addWidget(self.uiMacAddrLabel, 5, 0, 1, 1)
        self.uiPortNameFormatLabel = QtWidgets.QLabel(self.uiNetworkTab)
        self.uiPortNameFormatLabel.setObjectName("uiPortNameFormatLabel")
        self.gridLayout_5.addWidget(self.uiPortNameFormatLabel, 2, 0, 1, 1)
        self.uiAdaptersSpinBox = QtWidgets.QSpinBox(self.uiNetworkTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiAdaptersSpinBox.sizePolicy().hasHeightForWidth())
        self.uiAdaptersSpinBox.setSizePolicy(sizePolicy)
        self.uiAdaptersSpinBox.setMinimum(0)
        self.uiAdaptersSpinBox.setMaximum(275)
        self.uiAdaptersSpinBox.setObjectName("uiAdaptersSpinBox")
        self.gridLayout_5.addWidget(self.uiAdaptersSpinBox, 0, 1, 1, 1)
        self.uiPortNameFormatLineEdit = QtWidgets.QLineEdit(self.uiNetworkTab)
        self.uiPortNameFormatLineEdit.setText("")
        self.uiPortNameFormatLineEdit.setObjectName("uiPortNameFormatLineEdit")
        self.gridLayout_5.addWidget(self.uiPortNameFormatLineEdit, 2, 1, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(20, 261, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_5.addItem(spacerItem3, 8, 1, 1, 1)
        self.uiFirstPortNameLabel = QtWidgets.QLabel(self.uiNetworkTab)
        self.uiFirstPortNameLabel.setObjectName("uiFirstPortNameLabel")
        self.gridLayout_5.addWidget(self.uiFirstPortNameLabel, 1, 0, 1, 1)
        self.uiFirstPortNameLineEdit = QtWidgets.QLineEdit(self.uiNetworkTab)
        self.uiFirstPortNameLineEdit.setObjectName("uiFirstPortNameLineEdit")
        self.gridLayout_5.addWidget(self.uiFirstPortNameLineEdit, 1, 1, 1, 1)
        self.uiQemutabWidget.addTab(self.uiNetworkTab, "")
        self.uiAdvancedSettingsTab = QtWidgets.QWidget()
        self.uiAdvancedSettingsTab.setObjectName("uiAdvancedSettingsTab")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.uiAdvancedSettingsTab)
        self.verticalLayout_2.setContentsMargins(10, 10, 10, 10)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.uiLinuxBootGroupBox = QtWidgets.QGroupBox(self.uiAdvancedSettingsTab)
        self.uiLinuxBootGroupBox.setObjectName("uiLinuxBootGroupBox")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.uiLinuxBootGroupBox)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.uiKernelImageLineEdit = QtWidgets.QLineEdit(self.uiLinuxBootGroupBox)
        self.uiKernelImageLineEdit.setObjectName("uiKernelImageLineEdit")
        self.gridLayout_2.addWidget(self.uiKernelImageLineEdit, 1, 1, 1, 1)
        self.uiKernelCommandLineLabel = QtWidgets.QLabel(self.uiLinuxBootGroupBox)
        self.uiKernelCommandLineLabel.setObjectName("uiKernelCommandLineLabel")
        self.gridLayout_2.addWidget(self.uiKernelCommandLineLabel, 2, 0, 1, 1)
        self.uiInitrdLabel = QtWidgets.QLabel(self.uiLinuxBootGroupBox)
        self.uiInitrdLabel.setObjectName("uiInitrdLabel")
        self.gridLayout_2.addWidget(self.uiInitrdLabel, 0, 0, 1, 1)
        self.uiKernelImageLabel = QtWidgets.QLabel(self.uiLinuxBootGroupBox)
        self.uiKernelImageLabel.setObjectName("uiKernelImageLabel")
        self.gridLayout_2.addWidget(self.uiKernelImageLabel, 1, 0, 1, 1)
        self.uiInitrdLineEdit = QtWidgets.QLineEdit(self.uiLinuxBootGroupBox)
        self.uiInitrdLineEdit.setObjectName("uiInitrdLineEdit")
        self.gridLayout_2.addWidget(self.uiInitrdLineEdit, 0, 1, 1, 1)
        self.uiInitrdToolButton = QtWidgets.QToolButton(self.uiLinuxBootGroupBox)
        self.uiInitrdToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiInitrdToolButton.setObjectName("uiInitrdToolButton")
        self.gridLayout_2.addWidget(self.uiInitrdToolButton, 0, 2, 1, 1)
        self.uiKernelImageToolButton = QtWidgets.QToolButton(self.uiLinuxBootGroupBox)
        self.uiKernelImageToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiKernelImageToolButton.setObjectName("uiKernelImageToolButton")
        self.gridLayout_2.addWidget(self.uiKernelImageToolButton, 1, 2, 1, 1)
        self.uiKernelCommandLineEdit = QtWidgets.QLineEdit(self.uiLinuxBootGroupBox)
        self.uiKernelCommandLineEdit.setObjectName("uiKernelCommandLineEdit")
        self.gridLayout_2.addWidget(self.uiKernelCommandLineEdit, 2, 1, 1, 2)
        self.verticalLayout_2.addWidget(self.uiLinuxBootGroupBox)
        self.uiBiosImageGroupBox = QtWidgets.QGroupBox(self.uiAdvancedSettingsTab)
        self.uiBiosImageGroupBox.setObjectName("uiBiosImageGroupBox")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.uiBiosImageGroupBox)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.uiBiosLabel = QtWidgets.QLabel(self.uiBiosImageGroupBox)
        self.uiBiosLabel.setObjectName("uiBiosLabel")
        self.horizontalLayout.addWidget(self.uiBiosLabel)
        self.uiBiosImageLineEdit = QtWidgets.QLineEdit(self.uiBiosImageGroupBox)
        self.uiBiosImageLineEdit.setObjectName("uiBiosImageLineEdit")
        self.horizontalLayout.addWidget(self.uiBiosImageLineEdit)
        self.uiBiosImageToolButton = QtWidgets.QToolButton(self.uiBiosImageGroupBox)
        self.uiBiosImageToolButton.setObjectName("uiBiosImageToolButton")
        self.horizontalLayout.addWidget(self.uiBiosImageToolButton)
        self.verticalLayout_2.addWidget(self.uiBiosImageGroupBox)
        self.uiOptimizationGroupBox = QtWidgets.QGroupBox(self.uiAdvancedSettingsTab)
        self.uiOptimizationGroupBox.setObjectName("uiOptimizationGroupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.uiOptimizationGroupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.uiActivateCPUThrottlingCheckBox = QtWidgets.QCheckBox(self.uiOptimizationGroupBox)
        self.uiActivateCPUThrottlingCheckBox.setChecked(True)
        self.uiActivateCPUThrottlingCheckBox.setObjectName("uiActivateCPUThrottlingCheckBox")
        self.gridLayout.addWidget(self.uiActivateCPUThrottlingCheckBox, 0, 0, 1, 2)
        self.uiCPUThrottlingLabel = QtWidgets.QLabel(self.uiOptimizationGroupBox)
        self.uiCPUThrottlingLabel.setObjectName("uiCPUThrottlingLabel")
        self.gridLayout.addWidget(self.uiCPUThrottlingLabel, 1, 0, 1, 1)
        self.uiCPUThrottlingSpinBox = QtWidgets.QSpinBox(self.uiOptimizationGroupBox)
        self.uiCPUThrottlingSpinBox.setMinimum(1)
        self.uiCPUThrottlingSpinBox.setMaximum(800)
        self.uiCPUThrottlingSpinBox.setProperty("value", 100)
        self.uiCPUThrottlingSpinBox.setObjectName("uiCPUThrottlingSpinBox")
        self.gridLayout.addWidget(self.uiCPUThrottlingSpinBox, 1, 1, 1, 1)
        self.uiProcessPriorityLabel = QtWidgets.QLabel(self.uiOptimizationGroupBox)
        self.uiProcessPriorityLabel.setObjectName("uiProcessPriorityLabel")
        self.gridLayout.addWidget(self.uiProcessPriorityLabel, 2, 0, 1, 1)
        self.uiProcessPriorityComboBox = QtWidgets.QComboBox(self.uiOptimizationGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiProcessPriorityComboBox.sizePolicy().hasHeightForWidth())
        self.uiProcessPriorityComboBox.setSizePolicy(sizePolicy)
        self.uiProcessPriorityComboBox.setObjectName("uiProcessPriorityComboBox")
        self.uiProcessPriorityComboBox.addItem("")
        self.uiProcessPriorityComboBox.addItem("")
        self.uiProcessPriorityComboBox.addItem("")
        self.uiProcessPriorityComboBox.addItem("")
        self.uiProcessPriorityComboBox.addItem("")
        self.uiProcessPriorityComboBox.addItem("")
        self.gridLayout.addWidget(self.uiProcessPriorityComboBox, 2, 1, 1, 1)
        self.verticalLayout_2.addWidget(self.uiOptimizationGroupBox)
        self.groupBox = QtWidgets.QGroupBox(self.uiAdvancedSettingsTab)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.uiQemuOptionsLabel = QtWidgets.QLabel(self.groupBox)
        self.uiQemuOptionsLabel.setObjectName("uiQemuOptionsLabel")
        self.gridLayout_3.addWidget(self.uiQemuOptionsLabel, 0, 0, 1, 1)
        self.uiQemuOptionsLineEdit = QtWidgets.QLineEdit(self.groupBox)
        self.uiQemuOptionsLineEdit.setObjectName("uiQemuOptionsLineEdit")
        self.gridLayout_3.addWidget(self.uiQemuOptionsLineEdit, 0, 1, 1, 1)
        self.uiBaseVMCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.uiBaseVMCheckBox.setEnabled(True)
        self.uiBaseVMCheckBox.setObjectName("uiBaseVMCheckBox")
        self.gridLayout_3.addWidget(self.uiBaseVMCheckBox, 1, 0, 1, 2)
        self.uiACPIShutdownCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.uiACPIShutdownCheckBox.setObjectName("uiACPIShutdownCheckBox")
        self.gridLayout_3.addWidget(self.uiACPIShutdownCheckBox, 2, 0, 1, 2)
        self.uiQemuOptionsLineEdit.raise_()
        self.uiQemuOptionsLabel.raise_()
        self.uiACPIShutdownCheckBox.raise_()
        self.uiBaseVMCheckBox.raise_()
        self.verticalLayout_2.addWidget(self.groupBox)
        spacerItem4 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem4)
        self.uiQemutabWidget.addTab(self.uiAdvancedSettingsTab, "")
        self.verticalLayout.addWidget(self.uiQemutabWidget)

        self.retranslateUi(QemuVMConfigPageWidget)
        self.uiQemutabWidget.setCurrentIndex(0)
        self.uiProcessPriorityComboBox.setCurrentIndex(3)
        QtCore.QMetaObject.connectSlotsByName(QemuVMConfigPageWidget)

    def retranslateUi(self, QemuVMConfigPageWidget):
        _translate = QtCore.QCoreApplication.translate
        QemuVMConfigPageWidget.setWindowTitle(_translate("QemuVMConfigPageWidget", "QEMU VM configuration"))
        self.uiRamLabel.setText(_translate("QemuVMConfigPageWidget", "RAM:"))
        self.uiSymbolToolButton.setText(_translate("QemuVMConfigPageWidget", "&Browse..."))
        self.uiRamSpinBox.setSuffix(_translate("QemuVMConfigPageWidget", " MB"))
        self.uiSymbolLabel.setText(_translate("QemuVMConfigPageWidget", "Symbol:"))
        self.uiConsoleTypeComboBox.setItemText(0, _translate("QemuVMConfigPageWidget", "telnet"))
        self.uiConsoleTypeComboBox.setItemText(1, _translate("QemuVMConfigPageWidget", "vnc"))
        self.uiConsoleTypeComboBox.setItemText(2, _translate("QemuVMConfigPageWidget", "spice"))
        self.uiConsoleTypeLabel.setText(_translate("QemuVMConfigPageWidget", "Console type:"))
        self.uiBootPriorityLabel.setText(_translate("QemuVMConfigPageWidget", "Boot priority:"))
        self.uiQemuListLabel.setText(_translate("QemuVMConfigPageWidget", "Qemu binary:"))
        self.uiCategoryLabel.setText(_translate("QemuVMConfigPageWidget", "Category:"))
        self.uiNameLabel.setText(_translate("QemuVMConfigPageWidget", "Name:"))
        self.uiCPULabel.setText(_translate("QemuVMConfigPageWidget", "vCPUs:"))
        self.uiDefaultNameFormatLabel.setText(_translate("QemuVMConfigPageWidget", "Default name format:"))
        self.uiQemutabWidget.setTabText(self.uiQemutabWidget.indexOf(self.uiGeneralSettingsTab), _translate("QemuVMConfigPageWidget", "General settings"))
        self.uiHdaGroupBox.setTitle(_translate("QemuVMConfigPageWidget", "HDA (Primary Master)"))
        self.uiHdaDiskImageLabel.setText(_translate("QemuVMConfigPageWidget", "Disk image:"))
        self.uiHdaDiskImageToolButton.setText(_translate("QemuVMConfigPageWidget", "&Browse..."))
        self.uiHdaDiskImageCreateToolButton.setText(_translate("QemuVMConfigPageWidget", "Create..."))
        self.uiHdaDiskInterfaceLabel.setText(_translate("QemuVMConfigPageWidget", "Disk interface:"))
        self.uiHdbgroupBox.setTitle(_translate("QemuVMConfigPageWidget", "HDB (Primary Slave)"))
        self.uiHdbDiskImageLabel.setText(_translate("QemuVMConfigPageWidget", "Disk image:"))
        self.uiHdbDiskImageToolButton.setText(_translate("QemuVMConfigPageWidget", "&Browse..."))
        self.uiHdbDiskImageCreateToolButton.setText(_translate("QemuVMConfigPageWidget", "Create..."))
        self.uiHdbDiskInterfaceLabel.setText(_translate("QemuVMConfigPageWidget", "Disk interface:"))
        self.uiHdcGroupBox.setTitle(_translate("QemuVMConfigPageWidget", "HDC (Secondary Master)"))
        self.uiHdcDiskImageLabel.setText(_translate("QemuVMConfigPageWidget", "Disk image:"))
        self.uiHdcDiskImageToolButton.setText(_translate("QemuVMConfigPageWidget", "&Browse..."))
        self.uiHdcDiskImageCreateToolButton.setText(_translate("QemuVMConfigPageWidget", "Create..."))
        self.uiHdcDiskInterfaceLabel.setText(_translate("QemuVMConfigPageWidget", "Disk interface:"))
        self.uiHddGroupBox.setTitle(_translate("QemuVMConfigPageWidget", "HDD (Secondary Slave)"))
        self.uiHddDiskImageLabel.setText(_translate("QemuVMConfigPageWidget", "Disk image:"))
        self.uiHddDiskImageToolButton.setText(_translate("QemuVMConfigPageWidget", "&Browse..."))
        self.uiHddDiskImageCreateToolButton.setText(_translate("QemuVMConfigPageWidget", "Create..."))
        self.uiHddDiskInterfaceLabel.setText(_translate("QemuVMConfigPageWidget", "Disk interface:"))
        self.uiQemutabWidget.setTabText(self.uiQemutabWidget.indexOf(self.uiHddTab), _translate("QemuVMConfigPageWidget", "HDD"))
        self.uiCdromGroupBox.setTitle(_translate("QemuVMConfigPageWidget", "CD/DVD-ROM"))
        self.uiCdromImageLabel.setText(_translate("QemuVMConfigPageWidget", "Image:"))
        self.uiCdromImageToolButton.setText(_translate("QemuVMConfigPageWidget", "&Browse..."))
        self.uiQemutabWidget.setTabText(self.uiQemutabWidget.indexOf(self.uiCdromTab), _translate("QemuVMConfigPageWidget", "CD/DVD"))
        self.uiLegacyNetworkingCheckBox.setText(_translate("QemuVMConfigPageWidget", "Use the legacy networking mode"))
        self.uiPortSegmentSizeLabel.setText(_translate("QemuVMConfigPageWidget", "Segment size:"))
        self.uiAdapterTypesLabel.setText(_translate("QemuVMConfigPageWidget", "Type:"))
        self.uiAdaptersLabel.setText(_translate("QemuVMConfigPageWidget", "Adapters:"))
        self.uiMacAddrLabel.setText(_translate("QemuVMConfigPageWidget", "Base MAC:"))
        self.uiPortNameFormatLabel.setToolTip(_translate("QemuVMConfigPageWidget", "<html><head/><body><p>{0} - the port number, from 0 to the number of adapters-1.</p><p>{1} - the segment number, from 0 to the number of segments-1.</p><p>{port0} - named alias for {0}.</p><p>{port1} - the port number, from 1 to the number of adapters.</p><p>{segment0} - named alias for {1}.</p><p>{segment1} - the segment number, from 1 to the number of segments.</p></body></html>"))
        self.uiPortNameFormatLabel.setText(_translate("QemuVMConfigPageWidget", "Name format:"))
        self.uiFirstPortNameLabel.setText(_translate("QemuVMConfigPageWidget", "First port name:"))
        self.uiQemutabWidget.setTabText(self.uiQemutabWidget.indexOf(self.uiNetworkTab), _translate("QemuVMConfigPageWidget", "Network"))
        self.uiLinuxBootGroupBox.setTitle(_translate("QemuVMConfigPageWidget", "Linux boot specific settings"))
        self.uiKernelCommandLineLabel.setText(_translate("QemuVMConfigPageWidget", "Kernel command line:"))
        self.uiInitrdLabel.setText(_translate("QemuVMConfigPageWidget", "Initial RAM disk (initrd):"))
        self.uiKernelImageLabel.setText(_translate("QemuVMConfigPageWidget", "Kernel image:"))
        self.uiInitrdToolButton.setText(_translate("QemuVMConfigPageWidget", "&Browse..."))
        self.uiKernelImageToolButton.setText(_translate("QemuVMConfigPageWidget", "&Browse..."))
        self.uiBiosImageGroupBox.setTitle(_translate("QemuVMConfigPageWidget", "Bios"))
        self.uiBiosLabel.setText(_translate("QemuVMConfigPageWidget", "Bios image:"))
        self.uiBiosImageToolButton.setText(_translate("QemuVMConfigPageWidget", "Browse..."))
        self.uiOptimizationGroupBox.setTitle(_translate("QemuVMConfigPageWidget", "Optimizations"))
        self.uiActivateCPUThrottlingCheckBox.setText(_translate("QemuVMConfigPageWidget", "Activate CPU throttling"))
        self.uiCPUThrottlingLabel.setText(_translate("QemuVMConfigPageWidget", "Percentage of CPU allowed:"))
        self.uiCPUThrottlingSpinBox.setSuffix(_translate("QemuVMConfigPageWidget", " %"))
        self.uiProcessPriorityLabel.setText(_translate("QemuVMConfigPageWidget", "Process priority:"))
        self.uiProcessPriorityComboBox.setItemText(0, _translate("QemuVMConfigPageWidget", "Realtime"))
        self.uiProcessPriorityComboBox.setItemText(1, _translate("QemuVMConfigPageWidget", "Very high"))
        self.uiProcessPriorityComboBox.setItemText(2, _translate("QemuVMConfigPageWidget", "High"))
        self.uiProcessPriorityComboBox.setItemText(3, _translate("QemuVMConfigPageWidget", "Normal"))
        self.uiProcessPriorityComboBox.setItemText(4, _translate("QemuVMConfigPageWidget", "Low"))
        self.uiProcessPriorityComboBox.setItemText(5, _translate("QemuVMConfigPageWidget", "Very low"))
        self.groupBox.setTitle(_translate("QemuVMConfigPageWidget", "Additional settings"))
        self.uiQemuOptionsLabel.setText(_translate("QemuVMConfigPageWidget", "Options:"))
        self.uiQemuOptionsLineEdit.setToolTip(_translate("QemuVMConfigPageWidget", "<html><head/><body><p>Variable replacements:</p>\n"
"<ul>\n"
"<li>%vm-name% =VM name</li>\n"
"<li>%vm-id% =VM ID</li>\n"
"<li>%project-id% = project ID</li>\n"
"<li>%project-path% = project path</li>\n"
"</ul>\n"
"</body></html>"))
        self.uiBaseVMCheckBox.setText(_translate("QemuVMConfigPageWidget", "Use as a linked base VM"))
        self.uiACPIShutdownCheckBox.setText(_translate("QemuVMConfigPageWidget", "Enable ACPI shutdown (experimental)"))
        self.uiQemutabWidget.setTabText(self.uiQemutabWidget.indexOf(self.uiAdvancedSettingsTab), _translate("QemuVMConfigPageWidget", "Advanced settings"))




#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ..qt import QtGui


def colorFromSvg(value):
    """
    Transform a color coming from a SVG file to a Qcolor
    """
    value = value.strip('#')
    if value == "":
        value = "000000"
    if len(value) == 6:  # If alpha channel is missing
        value = "ff" + value
    value = int(value, base=16)
    return QtGui.QColor.fromRgba(value)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for Dynamips ATM switches.
"""

import re
from gns3.qt import QtCore, QtWidgets
from ..ui.atm_switch_configuration_page_ui import Ui_atmSwitchConfigPageWidget


class ATMSwitchConfigurationPage(QtWidgets.QWidget, Ui_atmSwitchConfigPageWidget):

    """
    QWidget configuration page for ATM switches.
    """

    def __init__(self):

        super().__init__()
        self.setupUi(self)
        self._mapping = {}

        # connect slots
        self.uiAddPushButton.clicked.connect(self._addMappingSlot)
        self.uiDeletePushButton.clicked.connect(self._deleteMappingSlot)
        self.uiMappingTreeWidget.itemActivated.connect(self._mappingSelectedSlot)
        self.uiMappingTreeWidget.itemSelectionChanged.connect(self._mappingSelectionChangedSlot)

        # enable sorting
        self.uiMappingTreeWidget.sortByColumn(0, QtCore.Qt.AscendingOrder)
        self.uiMappingTreeWidget.setSortingEnabled(True)

    def _mappingSelectedSlot(self, item, column):
        """
        Loads a selected mapping from the tree widget.

        :param item: selected QTreeWidgetItem instance
        :param column: ignored
        """

        source = item.text(0)
        destination = item.text(1)
        mapping = re.compile(r"""^([0-9]*):([0-9]*):([0-9]*)$""")
        match_source_mapping = mapping.search(source)
        match_destination_mapping = mapping.search(destination)

        if match_source_mapping and match_destination_mapping:
            self.uiVPICheckBox.setCheckState(QtCore.Qt.Unchecked)
            (source_port, source_vpi, source_vci) = match_source_mapping.group(1, 2, 3)
            (destination_port, destination_vpi, destination_vci) = match_destination_mapping.group(1, 2, 3)
        else:
            self.uiVPICheckBox.setCheckState(QtCore.Qt.Checked)
            (source_port, source_vpi) = source.split(':')
            (destination_port, destination_vpi) = destination.split(':')
            source_vci = destination_vci = 0

        # source
        self.uiSourcePortSpinBox.setValue(int(source_port))
        self.uiSourceVPISpinBox.setValue(int(source_vpi))
        self.uiSourceVCISpinBox.setValue(int(source_vci))

        # destination
        self.uiDestinationPortSpinBox.setValue(int(destination_port))
        self.uiDestinationVPISpinBox.setValue(int(destination_vpi))
        self.uiDestinationVCISpinBox.setValue(int(destination_vci))

    def _mappingSelectionChangedSlot(self):
        """
        Enables the use of the delete button.
        """

        item = self.uiMappingTreeWidget.currentItem()
        if item is not None:
            self.uiDeletePushButton.setEnabled(True)
        else:
            self.uiDeletePushButton.setEnabled(False)

    def _addMappingSlot(self):
        """
        Adds a new mapping.
        """

        source_port = self.uiSourcePortSpinBox.value()
        source_vpi = self.uiSourceVPISpinBox.value()
        source_vci = self.uiSourceVCISpinBox.value()
        destination_port = self.uiDestinationPortSpinBox.value()
        destination_vpi = self.uiDestinationVPISpinBox.value()
        destination_vci = self.uiDestinationVCISpinBox.value()

        if self.uiVPICheckBox.checkState() == QtCore.Qt.Unchecked:
            source = "{port}:{vpi}:{vci}".format(port=source_port,
                                                 vpi=source_vpi,
                                                 vci=source_vci)

            destination = "{port}:{vpi}:{vci}".format(port=destination_port,
                                                      vpi=destination_vpi,
                                                      vci=destination_vci)
        else:
            source = "{port}:{vpi}".format(port=source_port, vpi=source_vpi)
            destination = "{port}:{vpi}".format(port=destination_port, vpi=destination_vpi)

        if source in self._mapping or destination in self._mapping:
            QtWidgets.QMessageBox.critical(self, self._node.name(), "Mapping already defined")
            return

        item = QtWidgets.QTreeWidgetItem(self.uiMappingTreeWidget)
        item.setText(0, source)
        item.setText(1, destination)
        self.uiMappingTreeWidget.addTopLevelItem(item)
        self.uiSourcePortSpinBox.setValue(source_port + 1)
        self.uiDestinationPortSpinBox.setValue(destination_port + 1)
        self._mapping[source] = destination

    def _deleteMappingSlot(self):
        """
        Deletes a mapping.
        """

        item = self.uiMappingTreeWidget.currentItem()
        if item:

            source = item.text(0)
            source_port = int(source.split(':')[0])
            destination = item.text(1)
            destination_port = int(destination.split(':')[0])

            # check that a link isn't connected to these ports
            # before we delete that mapping
            node_ports = self._node.ports()
            for node_port in node_ports:
                if (node_port.portNumber() == source_port or node_port.portNumber() == destination_port) and not node_port.isFree():
                    QtWidgets.QMessageBox.critical(self, self._node.name(), "A link is connected to port {}, please remove it first".format(node_port.name()))
                    return

            del self._mapping[source]
            self.uiMappingTreeWidget.takeTopLevelItem(self.uiMappingTreeWidget.indexOfTopLevelItem(item))

    def loadSettings(self, settings, node, group=False):
        """
        Loads the ATM switch settings.

        :param settings: the settings (dictionary)
        :param node: Node instance
        :param group: indicates the settings apply to a group
        """

        if not group:
            self.uiNameLineEdit.setText(settings["name"])
        else:
            self.uiNameLineEdit.setEnabled(False)

        self.uiMappingTreeWidget.clear()
        self._mapping = {}
        self._node = node

        for source, destination in settings["mappings"].items():
            item = QtWidgets.QTreeWidgetItem(self.uiMappingTreeWidget)
            item.setText(0, source)
            item.setText(1, destination)
            self.uiMappingTreeWidget.addTopLevelItem(item)
            self._mapping[source] = destination

        self.uiMappingTreeWidget.resizeColumnToContents(0)
        self.uiMappingTreeWidget.resizeColumnToContents(1)

    def saveSettings(self, settings, node, group=False):
        """
        Saves the ATM switch settings.

        :param settings: the settings (dictionary)
        :param node: Node instance
        :param group: indicates the settings apply to a group
        """

        if not group:
            # set the device name
            name = self.uiNameLineEdit.text()
            if not name:
                QtWidgets.QMessageBox.critical(self, "Name", "ATM switch name cannot be empty!")
            else:
                settings["name"] = name
        settings["mappings"] = self._mapping.copy()
        return settings



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pytest
import unittest.mock
import pathlib

from gns3.registry.image import Image
from gns3.image_upload_manager import ImageUploadManager


@pytest.fixture
def image():
    return Image('QEMU', 'test.img')


@pytest.fixture
def controller():
    return unittest.mock.MagicMock()

@pytest.fixture
def callback():
    return unittest.mock.MagicMock()


def test_direct_file_upload(image, controller, callback):
    manager = ImageUploadManager(image, controller, 'compute_id', callback, directFileUpload=True)
    manager.upload()
    controller.getEndpoint.assert_called_with(
        '/QEMU/images/test.img',
        'compute_id',
        manager._onLoadEndpointCallback,
        showProgress=False
    )

    with unittest.mock.patch('gns3.image_upload_manager.HTTPClient') as client:
        manager._onLoadEndpointCallback(dict(endpoint='/endpoint'))
        client.fromUrl.return_value.createHTTPQuery.assert_called_with(
            'POST', '/endpoint', manager._checkIfSuccessfulCallback, body=pathlib.Path('test.img'),
            prefix="", progressText='Uploading test.img', timeout=None
        )

    manager._checkIfSuccessfulCallback({})
    callback.assert_called_with({}, False)


def test_direct_file_upload_fallback_to_controller(image, controller, callback):
    manager = ImageUploadManager(image, controller, callback, directFileUpload=True)
    manager._checkIfSuccessfulCallback({}, error=True, connection_error=True)
    controller.postCompute.assert_called_with(
        '/QEMU/images/test.img',
        callback,
        None,
        body=pathlib.Path('test.img'),
        progressText='Uploading test.img',
        timeout=None
    )


def test_upload_via_controller(image, controller, callback):
    manager = ImageUploadManager(image, controller, callback, directFileUpload=False)
    manager.upload()
    controller.postCompute.assert_called_with(
        '/QEMU/images/test.img',
        callback,
        None,
        body=pathlib.Path('test.img'),
        progressText='Uploading test.img',
        timeout=None
    )


# -*- coding: utf-8 -*-
#
# Copyright (C) 2017 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pathlib
import urllib.parse

from gns3.http_client import HTTPClient

import logging
log = logging.getLogger(__name__)


class ImageUploadManager(object):
    """
    Manager over the image upload. Encapsulates file uploads to computes or via controller.
    """

    def __init__(self, image, controller, compute_id, callback=None, directFileUpload=False):
        self._image = image
        self._compute_id = compute_id
        self._callback = callback
        self._directFileUpload = directFileUpload
        self._controller = controller

    def upload(self):
        if self._directFileUpload:
            # first obtain endpoint and know when target request
            self._controller.getEndpoint(
                self._getComputePath(), self._compute_id, self._onLoadEndpointCallback, showProgress=False)
        else:
            self._fileUploadToController()

    def _getComputePath(self):
        return '/{emulator}/images/{filename}'.format(
            emulator=self._image.emulator, filename=self._image.filename)

    def _onLoadEndpointCallback(self, result, error=False, **kwargs):
        if error:
            if "message" in result:
                log.error("Error while getting endpoint: {}".format(result["message"]))
            return

        # we know where is the endpoint and we trying to post there a file
        endpoint = result['endpoint']
        self._fileUploadToCompute(endpoint)

    def _checkIfSuccessfulCallback(self, result, error=False, **kwargs):
        if error:
            connection_error = kwargs.get('connection_error', False)
            if connection_error:
                log.debug("During direct file upload compute is not visible. Fallback to upload via controller.")
                # there was an issue with connection, probably we don't have a direct access to compute
                # we need to fallback to uploading files via controller
                self._fileUploadToController()
            else:
                if "message" in result:
                    log.error("Error while direct file upload: {}".format(result["message"]))
            return
        self._callback(result, error, **kwargs)

    def _fileUploadToCompute(self, endpoint):
        log.info("Uploading file to compute: {}".format(endpoint))

        parse_results = urllib.parse.urlparse(endpoint)
        network_manager = self._controller.getHttpClient().getNetworkManager()
        client = HTTPClient.fromUrl(endpoint, network_manager=network_manager)
        # We don't retry connection as in case of fail we try direct file upload
        client.setMaxRetryConnection(0)
        client.createHTTPQuery(
            'POST', parse_results.path, self._checkIfSuccessfulCallback, body=pathlib.Path(self._image.path),
            progressText="Uploading {}".format(self._image.filename), timeout=None, prefix="")

    def _fileUploadToController(self):
        log.info("Uploading file to controller: {}".format(self._getComputePath()))
        self._controller.postCompute(
            self._getComputePath(), self._compute_id, self._callback, body=pathlib.Path(self._image.path),
            progressText="Uploading {}".format(self._image.filename), timeout=None)


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/iou/ui/iou_device_wizard.ui'
#
# Created: Tue Sep 20 17:45:46 2016
#      by: PyQt5 UI code generator 5.2.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_IOUDeviceWizard(object):
    def setupUi(self, IOUDeviceWizard):
        IOUDeviceWizard.setObjectName("IOUDeviceWizard")
        IOUDeviceWizard.resize(586, 411)
        IOUDeviceWizard.setModal(True)
        self.uiServerWizardPage = QtWidgets.QWizardPage()
        self.uiServerWizardPage.setObjectName("uiServerWizardPage")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.uiServerWizardPage)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.uiServerTypeGroupBox = QtWidgets.QGroupBox(self.uiServerWizardPage)
        self.uiServerTypeGroupBox.setObjectName("uiServerTypeGroupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.uiServerTypeGroupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiRemoteRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        self.uiRemoteRadioButton.setChecked(True)
        self.uiRemoteRadioButton.setObjectName("uiRemoteRadioButton")
        self.verticalLayout.addWidget(self.uiRemoteRadioButton)
        self.uiVMRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        self.uiVMRadioButton.setObjectName("uiVMRadioButton")
        self.verticalLayout.addWidget(self.uiVMRadioButton)
        self.uiLocalRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        self.uiLocalRadioButton.setObjectName("uiLocalRadioButton")
        self.verticalLayout.addWidget(self.uiLocalRadioButton)
        self.gridLayout_2.addWidget(self.uiServerTypeGroupBox, 0, 0, 1, 1)
        self.uiRemoteServersGroupBox = QtWidgets.QGroupBox(self.uiServerWizardPage)
        self.uiRemoteServersGroupBox.setObjectName("uiRemoteServersGroupBox")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.uiRemoteServersGroupBox)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.uiRemoteServersLabel = QtWidgets.QLabel(self.uiRemoteServersGroupBox)
        self.uiRemoteServersLabel.setObjectName("uiRemoteServersLabel")
        self.gridLayout_7.addWidget(self.uiRemoteServersLabel, 0, 0, 1, 1)
        self.uiRemoteServersComboBox = QtWidgets.QComboBox(self.uiRemoteServersGroupBox)
        self.uiRemoteServersComboBox.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemoteServersComboBox.sizePolicy().hasHeightForWidth())
        self.uiRemoteServersComboBox.setSizePolicy(sizePolicy)
        self.uiRemoteServersComboBox.setObjectName("uiRemoteServersComboBox")
        self.gridLayout_7.addWidget(self.uiRemoteServersComboBox, 0, 1, 1, 1)
        self.gridLayout_2.addWidget(self.uiRemoteServersGroupBox, 1, 0, 1, 1)
        IOUDeviceWizard.addPage(self.uiServerWizardPage)
        self.uiNameWizardPage = QtWidgets.QWizardPage()
        self.uiNameWizardPage.setObjectName("uiNameWizardPage")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.uiNameWizardPage)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.formLayout_7 = QtWidgets.QFormLayout()
        self.formLayout_7.setFieldGrowthPolicy(QtWidgets.QFormLayout.ExpandingFieldsGrow)
        self.formLayout_7.setObjectName("formLayout_7")
        self.uiNameLabel = QtWidgets.QLabel(self.uiNameWizardPage)
        self.uiNameLabel.setObjectName("uiNameLabel")
        self.formLayout_7.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.uiNameLabel)
        self.uiNameLineEdit = QtWidgets.QLineEdit(self.uiNameWizardPage)
        self.uiNameLineEdit.setObjectName("uiNameLineEdit")
        self.formLayout_7.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.uiNameLineEdit)
        self.verticalLayout_2.addLayout(self.formLayout_7)
        self.groupBox = QtWidgets.QGroupBox(self.uiNameWizardPage)
        self.groupBox.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.groupBox.setAutoFillBackground(False)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.uiExistingImageRadioButton = QtWidgets.QRadioButton(self.groupBox)
        self.uiExistingImageRadioButton.setChecked(True)
        self.uiExistingImageRadioButton.setObjectName("uiExistingImageRadioButton")
        self.horizontalLayout_2.addWidget(self.uiExistingImageRadioButton)
        self.uiNewImageRadioButton = QtWidgets.QRadioButton(self.groupBox)
        self.uiNewImageRadioButton.setChecked(False)
        self.uiNewImageRadioButton.setObjectName("uiNewImageRadioButton")
        self.horizontalLayout_2.addWidget(self.uiNewImageRadioButton)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.formLayout_8 = QtWidgets.QFormLayout()
        self.formLayout_8.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout_8.setObjectName("formLayout_8")
        self.uiTypeLabel = QtWidgets.QLabel(self.groupBox)
        self.uiTypeLabel.setObjectName("uiTypeLabel")
        self.formLayout_8.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.uiTypeLabel)
        self.uiTypeComboBox = QtWidgets.QComboBox(self.groupBox)
        self.uiTypeComboBox.setObjectName("uiTypeComboBox")
        self.formLayout_8.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.uiTypeComboBox)
        self.uiIOUImageLabel = QtWidgets.QLabel(self.groupBox)
        self.uiIOUImageLabel.setObjectName("uiIOUImageLabel")
        self.formLayout_8.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.uiIOUImageLabel)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.uiIOUImageListComboBox = QtWidgets.QComboBox(self.groupBox)
        self.uiIOUImageListComboBox.setObjectName("uiIOUImageListComboBox")
        self.horizontalLayout_5.addWidget(self.uiIOUImageListComboBox)
        self.uiIOUImageLineEdit = QtWidgets.QLineEdit(self.groupBox)
        self.uiIOUImageLineEdit.setObjectName("uiIOUImageLineEdit")
        self.horizontalLayout_5.addWidget(self.uiIOUImageLineEdit)
        self.uiIOUImageToolButton = QtWidgets.QToolButton(self.groupBox)
        self.uiIOUImageToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiIOUImageToolButton.setObjectName("uiIOUImageToolButton")
        self.horizontalLayout_5.addWidget(self.uiIOUImageToolButton)
        self.formLayout_8.setLayout(2, QtWidgets.QFormLayout.FieldRole, self.horizontalLayout_5)
        self.verticalLayout_3.addLayout(self.formLayout_8)
        self.verticalLayout_2.addWidget(self.groupBox)
        IOUDeviceWizard.addPage(self.uiNameWizardPage)

        self.retranslateUi(IOUDeviceWizard)
        QtCore.QMetaObject.connectSlotsByName(IOUDeviceWizard)

    def retranslateUi(self, IOUDeviceWizard):
        _translate = QtCore.QCoreApplication.translate
        IOUDeviceWizard.setWindowTitle(_translate("IOUDeviceWizard", "New IOU device template"))
        self.uiServerWizardPage.setTitle(_translate("IOUDeviceWizard", "Server"))
        self.uiServerWizardPage.setSubTitle(_translate("IOUDeviceWizard", "Please choose a server type to run your new IOU device."))
        self.uiServerTypeGroupBox.setTitle(_translate("IOUDeviceWizard", "Server type"))
        self.uiRemoteRadioButton.setText(_translate("IOUDeviceWizard", "Run this IOU device on a remote computers"))
        self.uiVMRadioButton.setText(_translate("IOUDeviceWizard", "Run this IOU device on the GNS3 VM"))
        self.uiLocalRadioButton.setText(_translate("IOUDeviceWizard", "Run this IOU device on my local computer"))
        self.uiRemoteServersGroupBox.setTitle(_translate("IOUDeviceWizard", "Remote server"))
        self.uiRemoteServersLabel.setText(_translate("IOUDeviceWizard", "Run on:"))
        self.uiNameWizardPage.setTitle(_translate("IOUDeviceWizard", "Name and image"))
        self.uiNameWizardPage.setSubTitle(_translate("IOUDeviceWizard", "Please choose a descriptive name for the new IOU device and add an IOU image."))
        self.uiNameLabel.setText(_translate("IOUDeviceWizard", "Name:"))
        self.groupBox.setTitle(_translate("IOUDeviceWizard", "Image"))
        self.uiExistingImageRadioButton.setText(_translate("IOUDeviceWizard", "Existing image"))
        self.uiNewImageRadioButton.setText(_translate("IOUDeviceWizard", "New Image"))
        self.uiTypeLabel.setText(_translate("IOUDeviceWizard", "Type:"))
        self.uiIOUImageLabel.setText(_translate("IOUDeviceWizard", "IOU image:"))
        self.uiIOUImageToolButton.setText(_translate("IOUDeviceWizard", "&Browse..."))




# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Dynamips module implementation.
"""

import os
import shutil
import hashlib

from gns3.qt import QtWidgets
from gns3.local_config import LocalConfig
from gns3.image_manager import ImageManager
from gns3.local_server_config import LocalServerConfig

from ..module import Module
from ..module_error import ModuleError
from .nodes.router import Router
from .nodes.c1700 import C1700
from .nodes.c2600 import C2600
from .nodes.c2691 import C2691
from .nodes.c3600 import C3600
from .nodes.c3725 import C3725
from .nodes.c3745 import C3745
from .nodes.c7200 import C7200
from .nodes.etherswitch_router import EtherSwitchRouter
from .settings import DYNAMIPS_SETTINGS
from .settings import IOS_ROUTER_SETTINGS
from .settings import DEFAULT_IDLEPC

PLATFORM_TO_CLASS = {
    "c1700": C1700,
    "c2600": C2600,
    "c2691": C2691,
    "c3600": C3600,
    "c3725": C3725,
    "c3745": C3745,
    "c7200": C7200
}

import logging
log = logging.getLogger(__name__)


class Dynamips(Module):

    """
    Dynamips module.
    """

    def __init__(self):
        super().__init__()

        self._settings = {}
        self._ios_routers = {}
        self._nodes = []
        self._ios_images_cache = {}

        self.configChangedSlot()

    def configChangedSlot(self):
        # load the settings and IOS images.
        self._loadSettings()

    @staticmethod
    def getDefaultIdlePC(path):
        """
        Return the default IDLE PC for an image if the image
        exists or None otherwise
        """
        if not os.path.isfile(path):
            path = os.path.join(ImageManager.instance().getDirectoryForType("DYNAMIPS"), path)
            if not os.path.isfile(path):
                return None
        try:
            md5sum = Dynamips._md5sum(path)
            log.debug("Get idlePC for %s. md5sum %s", path, md5sum)
            if md5sum in DEFAULT_IDLEPC:
                log.debug("IDLEPC found for %s", path)
                return DEFAULT_IDLEPC[md5sum]
        except OSError:
            return None

    @staticmethod
    def _md5sum(path):
        with open(path, "rb") as fd:
            m = hashlib.md5()
            while True:
                data = fd.read(8192)
                if not data:
                    break
                m.update(data)
            return m.hexdigest()

    def _loadSettings(self):
        """
        Loads the settings from the persistent settings file.
        """

        self._settings = LocalConfig.instance().loadSectionSettings(self.__class__.__name__, DYNAMIPS_SETTINGS)
        if not os.path.exists(self._settings["dynamips_path"]):
            dynamips_path = shutil.which("dynamips")
            if dynamips_path:
                self._settings["dynamips_path"] = os.path.abspath(dynamips_path)
            else:
                self._settings["dynamips_path"] = ""

        self._loadIOSRouters()

    def _saveSettings(self):
        """
        Saves the settings to the persistent settings file.
        """

        # save the settings
        LocalConfig.instance().saveSectionSettings(self.__class__.__name__, self._settings)

        # save some settings to the local server config file
        server_settings = {
            "allocate_aux_console_ports": self._settings["allocate_aux_console_ports"],
            "ghost_ios_support": self._settings["ghost_ios_support"],
            "sparse_memory_support": self._settings["sparse_memory_support"],
            "mmap_support": self._settings["mmap_support"],
        }

        if self._settings["dynamips_path"]:
            server_settings["dynamips_path"] = os.path.normpath(self._settings["dynamips_path"])
        config = LocalServerConfig.instance()
        config.saveSettings(self.__class__.__name__, server_settings)

    def _loadIOSRouters(self):
        """
        Load the IOS routers from the persistent settings file.
        """

        self._ios_routers = {}
        settings = LocalConfig.instance().settings()
        if "routers" in settings.get(self.__class__.__name__, {}):
            for router in settings[self.__class__.__name__]["routers"]:
                name = router.get("name")
                server = router.get("server")
                router["image"] = router.get("path", router["image"])  # for backward compatibility before version 1.3
                key = "{server}:{name}".format(server=server, name=name)
                if key in self._ios_routers or not name or not server:
                    continue
                router_settings = IOS_ROUTER_SETTINGS.copy()
                router_settings.update(router)
                # for backward compatibility before version 1.4
                if "symbol" not in router_settings:
                    router_settings["symbol"] = router_settings["default_symbol"]
                    router_settings["symbol"] = router_settings["symbol"][:-11] + ".svg" if router_settings["symbol"].endswith("normal.svg") else router_settings["symbol"]
                self._ios_routers[key] = router_settings

    def _saveIOSRouters(self):
        """
        Saves the IOS routers to the persistent settings file.
        """

        self._settings["routers"] = list(self._ios_routers.values())
        self._saveSettings()

    def addNode(self, node):
        """
        Adds a node to this module.

        :param node: Node instance
        """

        self._nodes.append(node)

    def removeNode(self, node):
        """
        Removes a node from this module.

        :param node: Node instance
        """

        if node in self._nodes:
            self._nodes.remove(node)

    def VMs(self):
        """
        Returns IOS routers settings.

        :returns: IOS routers settings (dictionary)
        """

        return self._ios_routers

    def setVMs(self, new_ios_routers):
        """
        Sets IOS images settings.

        :param new_ios_routers: IOS images settings (dictionary)
        """

        self._ios_routers = new_ios_routers.copy()
        self._saveIOSRouters()

    @staticmethod
    def vmConfigurationPage():
        from .pages.ios_router_configuration_page import IOSRouterConfigurationPage
        return IOSRouterConfigurationPage

    def settings(self):
        """
        Returns the module settings

        :returns: module settings (dictionary)
        """

        return self._settings

    def setSettings(self, settings):
        """
        Sets the module settings

        :param settings: module settings (dictionary)
        """

        self._settings.update(settings)
        self._saveSettings()

    def instantiateNode(self, node_class, server, project):
        """
        Instantiate a new node.

        :param node_class: Node object
        :param server: HTTPClient instance
        :param project: Project instance
        """

        # create an instance of the node class
        return node_class(self, server, project)

    def updateImageIdlepc(self, image_path, idlepc):
        """
        Updates the Idle-PC for an IOS image.

        :param image_path: path to the IOS image
        :param idlepc: Idle-PC value
        """

        for ios_router in self._ios_routers.values():
            if os.path.basename(ios_router["image"]) == image_path:
                if ios_router["idlepc"] != idlepc:
                    ios_router["idlepc"] = idlepc
                    log.debug("Idle-PC value {} saved into '{}' template".format(idlepc, ios_router["name"]))
                    self._saveIOSRouters()

    def reset(self):
        """
        Resets the module.
        """

        self._nodes.clear()

    def findAlternativeIOSImage(self, image, node):
        """
        Tries to find an alternative IOS image.

        :param image: image name
        :param node: requesting Node instance

        :return: IOS image (dictionary)
        """

        if image in self._ios_images_cache:
            return self._ios_images_cache[image]

        from gns3.main_window import MainWindow
        mainwindow = MainWindow.instance()
        ios_routers = self.VMs()
        candidate_ios_images = {}
        alternative_image = {"image": image,
                             "ram": None,
                             "idlepc": None}

        # find all images with the same platform and local server
        for ios_router in ios_routers.values():
            if ios_router["platform"] == node.settings()["platform"] and ios_router["server"] == "local":
                if "chassis" in node.settings() and ios_router["chassis"] != node.settings()["chassis"]:
                    # continue to look if the chassis is not compatible
                    continue
                candidate_ios_images[ios_router["image"]] = ios_router

        if candidate_ios_images:
            selection, ok = QtWidgets.QInputDialog.getItem(mainwindow,
                                                           "IOS image", "IOS image {} could not be found\nPlease select an alternative from your existing images:".format(image),
                                                           list(candidate_ios_images.keys()), 0, False)
            if ok:
                candidate = candidate_ios_images[selection]
                alternative_image["image"] = candidate["image"]
                alternative_image["ram"] = candidate["ram"]
                alternative_image["idlepc"] = candidate["idlepc"]
                self._ios_images_cache[image] = alternative_image
                return alternative_image

        # no registered IOS image is used, let's just ask for an IOS image path
        msg = "Could not find the {} IOS image \nPlease select a similar IOS image!".format(image)
        log.error(msg)
        QtWidgets.QMessageBox.critical(mainwindow, "IOS image", msg)
        from .pages.ios_router_preferences_page import IOSRouterPreferencesPage
        image_path = IOSRouterPreferencesPage.getIOSImage(mainwindow, None)
        if image_path:
            alternative_image["image"] = image_path
            self._ios_images_cache[image] = alternative_image
        return alternative_image

    @staticmethod
    def getNodeClass(name):
        """
        Returns the object with the corresponding name.

        :param name: object name
        """

        if name in globals():
            return globals()[name]
        return None

    @staticmethod
    def getNodeType(name, platform=None):
        if name == "dynamips":
            return PLATFORM_TO_CLASS[platform]
        return None

    @staticmethod
    def classes():
        """
        Returns all the node classes supported by this module.

        :returns: list of classes
        """

        return [C1700, C2600, C2691, C3600, C3725, C3745, C7200, EtherSwitchRouter]

    def nodes(self):
        """
        Returns all the node data necessary to represent a node
        in the nodes view and create a node on the scene.
        """

        nodes = []
        for ios_router in self._ios_routers.values():
            node_class = PLATFORM_TO_CLASS[ios_router["platform"]]
            nodes.append(
                {"class": node_class.__name__,
                 "name": ios_router["name"],
                 "server": ios_router["server"],
                 "symbol": ios_router["symbol"],
                 "categories": [ios_router["category"]]}
            )

        return nodes

    @staticmethod
    def preferencePages():
        """
        :returns: QWidget object list
        """

        from .pages.dynamips_preferences_page import DynamipsPreferencesPage
        from .pages.ios_router_preferences_page import IOSRouterPreferencesPage
        return [DynamipsPreferencesPage, IOSRouterPreferencesPage]

    @staticmethod
    def instance():
        """
        Singleton to return only on instance of Dynamips.

        :returns: instance of Dynamips
        """

        if not hasattr(Dynamips, "_instance"):
            Dynamips._instance = Dynamips()
        return Dynamips._instance



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/gns3_vm_preferences_page.ui'
#
# Created: Sat Oct  8 14:06:27 2016
#      by: PyQt5 UI code generator 5.2.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_GNS3VMPreferencesPageWidget(object):
    def setupUi(self, GNS3VMPreferencesPageWidget):
        GNS3VMPreferencesPageWidget.setObjectName("GNS3VMPreferencesPageWidget")
        GNS3VMPreferencesPageWidget.resize(476, 450)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(GNS3VMPreferencesPageWidget.sizePolicy().hasHeightForWidth())
        GNS3VMPreferencesPageWidget.setSizePolicy(sizePolicy)
        GNS3VMPreferencesPageWidget.setMinimumSize(QtCore.QSize(0, 0))
        self.verticalLayout = QtWidgets.QVBoxLayout(GNS3VMPreferencesPageWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiEnableVMCheckBox = QtWidgets.QCheckBox(GNS3VMPreferencesPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiEnableVMCheckBox.sizePolicy().hasHeightForWidth())
        self.uiEnableVMCheckBox.setSizePolicy(sizePolicy)
        self.uiEnableVMCheckBox.setMinimumSize(QtCore.QSize(0, 0))
        self.uiEnableVMCheckBox.setChecked(False)
        self.uiEnableVMCheckBox.setObjectName("uiEnableVMCheckBox")
        self.verticalLayout.addWidget(self.uiEnableVMCheckBox)
        self.uiVirtualizationGroupBox = QtWidgets.QGroupBox(GNS3VMPreferencesPageWidget)
        self.uiVirtualizationGroupBox.setObjectName("uiVirtualizationGroupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.uiVirtualizationGroupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.uiGNS3VMEngineComboBox = QtWidgets.QComboBox(self.uiVirtualizationGroupBox)
        self.uiGNS3VMEngineComboBox.setObjectName("uiGNS3VMEngineComboBox")
        self.verticalLayout_2.addWidget(self.uiGNS3VMEngineComboBox)
        self.uiEngineDescriptionLabel = QtWidgets.QLabel(self.uiVirtualizationGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiEngineDescriptionLabel.sizePolicy().hasHeightForWidth())
        self.uiEngineDescriptionLabel.setSizePolicy(sizePolicy)
        self.uiEngineDescriptionLabel.setWordWrap(True)
        self.uiEngineDescriptionLabel.setOpenExternalLinks(True)
        self.uiEngineDescriptionLabel.setObjectName("uiEngineDescriptionLabel")
        self.verticalLayout_2.addWidget(self.uiEngineDescriptionLabel)
        self.verticalLayout.addWidget(self.uiVirtualizationGroupBox)
        self.uiGNS3VMSettingsGroupBox = QtWidgets.QGroupBox(GNS3VMPreferencesPageWidget)
        self.uiGNS3VMSettingsGroupBox.setObjectName("uiGNS3VMSettingsGroupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.uiGNS3VMSettingsGroupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.uiActionCloseLabel = QtWidgets.QLabel(self.uiGNS3VMSettingsGroupBox)
        self.uiActionCloseLabel.setObjectName("uiActionCloseLabel")
        self.gridLayout.addWidget(self.uiActionCloseLabel, 8, 0, 1, 2)
        self.uiHeadlessCheckBox = QtWidgets.QCheckBox(self.uiGNS3VMSettingsGroupBox)
        self.uiHeadlessCheckBox.setObjectName("uiHeadlessCheckBox")
        self.gridLayout.addWidget(self.uiHeadlessCheckBox, 1, 0, 1, 2)
        self.uiWhenExitKeepRadioButton = QtWidgets.QRadioButton(self.uiGNS3VMSettingsGroupBox)
        self.uiWhenExitKeepRadioButton.setObjectName("uiWhenExitKeepRadioButton")
        self.gridLayout.addWidget(self.uiWhenExitKeepRadioButton, 9, 0, 1, 2)
        self.uiWhenExitSuspendRadioButton = QtWidgets.QRadioButton(self.uiGNS3VMSettingsGroupBox)
        self.uiWhenExitSuspendRadioButton.setObjectName("uiWhenExitSuspendRadioButton")
        self.gridLayout.addWidget(self.uiWhenExitSuspendRadioButton, 10, 0, 1, 2)
        self.uiVMNameLabel = QtWidgets.QLabel(self.uiGNS3VMSettingsGroupBox)
        self.uiVMNameLabel.setObjectName("uiVMNameLabel")
        self.gridLayout.addWidget(self.uiVMNameLabel, 0, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.uiVMListComboBox = QtWidgets.QComboBox(self.uiGNS3VMSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiVMListComboBox.sizePolicy().hasHeightForWidth())
        self.uiVMListComboBox.setSizePolicy(sizePolicy)
        self.uiVMListComboBox.setObjectName("uiVMListComboBox")
        self.horizontalLayout.addWidget(self.uiVMListComboBox)
        self.uiRefreshPushButton = QtWidgets.QPushButton(self.uiGNS3VMSettingsGroupBox)
        self.uiRefreshPushButton.setObjectName("uiRefreshPushButton")
        self.horizontalLayout.addWidget(self.uiRefreshPushButton)
        self.gridLayout.addLayout(self.horizontalLayout, 0, 1, 1, 1)
        self.uiWhenExitStopRadioButton = QtWidgets.QRadioButton(self.uiGNS3VMSettingsGroupBox)
        self.uiWhenExitStopRadioButton.setObjectName("uiWhenExitStopRadioButton")
        self.gridLayout.addWidget(self.uiWhenExitStopRadioButton, 11, 0, 1, 2)
        self.uiRamLabel = QtWidgets.QLabel(self.uiGNS3VMSettingsGroupBox)
        self.uiRamLabel.setObjectName("uiRamLabel")
        self.gridLayout.addWidget(self.uiRamLabel, 4, 0, 1, 1)
        self.uiRamSpinBox = QtWidgets.QSpinBox(self.uiGNS3VMSettingsGroupBox)
        self.uiRamSpinBox.setMinimum(512)
        self.uiRamSpinBox.setMaximum(1000000)
        self.uiRamSpinBox.setSingleStep(512)
        self.uiRamSpinBox.setProperty("value", 2048)
        self.uiRamSpinBox.setObjectName("uiRamSpinBox")
        self.gridLayout.addWidget(self.uiRamSpinBox, 4, 1, 1, 1)
        self.uiCpuSpinBox = QtWidgets.QSpinBox(self.uiGNS3VMSettingsGroupBox)
        self.uiCpuSpinBox.setMinimum(1)
        self.uiCpuSpinBox.setProperty("value", 1)
        self.uiCpuSpinBox.setObjectName("uiCpuSpinBox")
        self.gridLayout.addWidget(self.uiCpuSpinBox, 5, 1, 1, 1)
        self.uiCpuLabel = QtWidgets.QLabel(self.uiGNS3VMSettingsGroupBox)
        self.uiCpuLabel.setObjectName("uiCpuLabel")
        self.gridLayout.addWidget(self.uiCpuLabel, 5, 0, 1, 1)
        self.verticalLayout.addWidget(self.uiGNS3VMSettingsGroupBox)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)

        self.retranslateUi(GNS3VMPreferencesPageWidget)
        QtCore.QMetaObject.connectSlotsByName(GNS3VMPreferencesPageWidget)

    def retranslateUi(self, GNS3VMPreferencesPageWidget):
        _translate = QtCore.QCoreApplication.translate
        GNS3VMPreferencesPageWidget.setWindowTitle(_translate("GNS3VMPreferencesPageWidget", "GNS3 VM"))
        self.uiEnableVMCheckBox.setText(_translate("GNS3VMPreferencesPageWidget", "Enable the GNS3 VM"))
        self.uiVirtualizationGroupBox.setTitle(_translate("GNS3VMPreferencesPageWidget", "Virtualization engine"))
        self.uiEngineDescriptionLabel.setText(_translate("GNS3VMPreferencesPageWidget", "Description"))
        self.uiGNS3VMSettingsGroupBox.setTitle(_translate("GNS3VMPreferencesPageWidget", "Settings"))
        self.uiActionCloseLabel.setText(_translate("GNS3VMPreferencesPageWidget", "Action when closing GNS3:"))
        self.uiHeadlessCheckBox.setText(_translate("GNS3VMPreferencesPageWidget", "Run the VM in headless mode"))
        self.uiWhenExitKeepRadioButton.setText(_translate("GNS3VMPreferencesPageWidget", "keep the GNS3 VM running"))
        self.uiWhenExitSuspendRadioButton.setText(_translate("GNS3VMPreferencesPageWidget", "suspend the GNS3 VM"))
        self.uiVMNameLabel.setText(_translate("GNS3VMPreferencesPageWidget", "VM name:"))
        self.uiRefreshPushButton.setText(_translate("GNS3VMPreferencesPageWidget", "&Refresh"))
        self.uiWhenExitStopRadioButton.setText(_translate("GNS3VMPreferencesPageWidget", "stop the GNS3 VM"))
        self.uiRamLabel.setText(_translate("GNS3VMPreferencesPageWidget", "RAM:"))
        self.uiRamSpinBox.setSuffix(_translate("GNS3VMPreferencesPageWidget", " MB"))
        self.uiCpuLabel.setText(_translate("GNS3VMPreferencesPageWidget", "vCPUs:"))




# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Topology summary view that list all the nodes, their status and connections.
"""

from .qt import QtGui, QtCore, QtWidgets, qslot
from .node import Node
from .topology import Topology
from .items.node_item import NodeItem
from .items.link_item import LinkItem
from .packet_capture import PacketCapture
from .utils import natural_sort_key

import logging
log = logging.getLogger(__name__)


class TopologyNodeItem(QtWidgets.QTreeWidgetItem):

    """
    Custom item for the QTreeWidget instance
    (topology summary view).

    :param parent: parent widget
    :param node: Node instance
    """

    def __init__(self, parent, node):

        super().__init__(parent)
        self._node = node
        self._parent = parent

        # we want to know about the node events
        node.started_signal.connect(self._refreshStatusSlot)
        node.stopped_signal.connect(self._refreshStatusSlot)
        node.suspended_signal.connect(self._refreshStatusSlot)
        node.updated_signal.connect(self._refreshNodeSlot)
        node.created_signal.connect(self._refreshNodeSlot)
        node.deleted_signal.connect(self._deletedNodeSlot)

        self._refreshStatusSlot()
        self._refreshNodeSlot()

    @qslot
    def _refreshStatusSlot(self, *args):
        """
        Changes the icon to show the node status (started, stopped etc.)
        """
        self.setText(0, self._node.name())
        if self._node.status() == Node.started:
            self.setIcon(0, QtGui.QIcon(':/icons/led_green.svg'))
        elif self._node.status() == Node.suspended:
            self.setIcon(0, QtGui.QIcon(':/icons/led_yellow.svg'))
        else:
            self.setIcon(0, QtGui.QIcon(':/icons/led_red.svg'))

    @qslot
    def _refreshNodeSlot(self, *args):
        """
        Slot to update the node.
        """
        self.refresh()

    def node(self):
        """
        Returns the node.

        :return: Node instance
        """

        return self._node

    def refresh(self):
        """
        Updates the widget item with the current node name.
        """

        if self._node.name() != self.text(0):
            # refresh all the other item if the node name has changed
            self._parent.refreshAllLinks(source_child=self)
        self.setText(0, self._node.name())
        if self._node.consoleType() and self._node.console():
            self.setText(1, "{} {}:{}".format(self._node.consoleType(), self._node.consoleHost(), self._node.console()))
        else:
            self.setText(1, "not supported")
        self.refreshLinks()
        self._parent.invisibleRootItem().sortChildren(0, QtCore.Qt.AscendingOrder)

    def refreshLinks(self):
        """
        List all the connections as children.
        """

        self.takeChildren()

        capturing = False
        filtering = False
        for link in self._node.links():
            item = QtWidgets.QTreeWidgetItem()
            port = link.getNodePort(self._node)
            item.setText(0, "{} {}".format(port.shortName(), port.description(short=True)))
            item.setData(0, QtCore.Qt.UserRole, link)
            if link.capturing():
                item.setIcon(0, QtGui.QIcon(':/icons/inspect.svg'))
                capturing = True
            if len(link.filters()) > 0:
                item.setIcon(0, QtGui.QIcon(':/icons/filter.svg'))
                filtering = True
            if link.capturing() and len(link.filters()) > 0:
                item.setIcon(0, QtGui.QIcon(':/icons/filter-capture.svg'))
            if link.suspended():
                item.setIcon(0, QtGui.QIcon(':/icons/pause.svg'))
            self.addChild(item)

        if self._parent.show_only_devices_with_capture and capturing is False:
            self.setHidden(True)
        elif self._parent.show_only_devices_with_filters and filtering is False:
            self.setHidden(True)
        else:
            self.setHidden(False)

        self.sortChildren(0, QtCore.Qt.AscendingOrder)

    @qslot
    def _deletedNodeSlot(self, *args):
        """
        Removes the node from the view.
        """

        tree = self.treeWidget()
        tree.takeTopLevelItem(tree.indexOfTopLevelItem(self))
        tree.nodes_id.remove(self._node.id())

    def __lt__(self, otherItem):
        column = self.treeWidget().sortColumn()
        return natural_sort_key(self.text(column)) < natural_sort_key(otherItem.text(column))


class TopologySummaryView(QtWidgets.QTreeWidget):

    """
    Topology summary view implementation.

    :param parent: parent widget
    """

    def __init__(self, parent):

        super().__init__(parent)
        self.nodes_id = set()
        self._topology = Topology.instance()
        self._topology.node_added_signal.connect(self._nodeAddedSlot)
        self._topology.project_changed_signal.connect(self._projectChangedSlot)
        self.itemSelectionChanged.connect(self._itemSelectionChangedSlot)
        self.show_only_devices_with_capture = False
        self.show_only_devices_with_filters = False
        self.setExpandsOnDoubleClick(False)
        self.itemDoubleClicked.connect(self._itemDoubleClickedSlot)

    @qslot
    def _projectChangedSlot(self, *args):
        """
        Clears all the topology summary.
        """

        self.clear()

    def refreshAllLinks(self, source_child=None):
        """
        Refreshes all links for all items.
        """

        root = self.invisibleRootItem()
        for index in range(0, root.childCount()):
            child = root.child(index)
            if source_child and source_child == child:
                continue
            child.refreshLinks()

    @qslot
    def _nodeAddedSlot(self, base_node_id, *args):
        """
        Received events for node creation.

        :param base_node_id: base node identifier
        """

        if not base_node_id:
            log.error("node ID is null")
            return

        node = self._topology.getNode(base_node_id)
        if not node:
            log.error("could not find node with ID {}".format(base_node_id))
            return

        # We check if we don't already have this node because it seem
        # sometimes we can get twice the signal
        if node.id() in self.nodes_id:
            return
        self.nodes_id.add(node.id())
        TopologyNodeItem(self, node)
        self.resizeColumnToContents(0)

    @qslot
    def _itemSelectionChangedSlot(self, *args):
        """
        Slot called when an item is selected in the TreeWidget.
        """

        current_item = self.currentItem()
        if current_item:
            from .main_window import MainWindow
            view = MainWindow.instance().uiGraphicsView
            for item in view.scene().items():
                if isinstance(item, NodeItem):
                    item.setSelected(False)
                    if isinstance(current_item, TopologyNodeItem) and item.node().id() == current_item.node().id():
                        item.setSelected(True)
                elif isinstance(item, LinkItem):
                    item.setHovered(False)
                    if not isinstance(current_item, TopologyNodeItem):
                        link = current_item.data(0, QtCore.Qt.UserRole)
                        if item.link() == link:
                            item.setHovered(True)

    @qslot
    def _itemDoubleClickedSlot(self, current_item, *args):
        """
        When user double click on an element we center the topology on it
        """
        if current_item != 0:
            from .main_window import MainWindow
            view = MainWindow.instance().uiGraphicsView
            for item in view.scene().items():
                if isinstance(item, NodeItem):
                    if isinstance(current_item, TopologyNodeItem) and item.node().id() == current_item.node().id():
                        view.centerOn(item)
                elif isinstance(item, LinkItem):
                    if not isinstance(current_item, TopologyNodeItem):
                        link = current_item.data(0, QtCore.Qt.UserRole)
                        if item.link() == link:
                            view.centerOn(item)

    def mousePressEvent(self, event):
        """
        Handles all mouse press events.

        :param event: QMouseEvent instance
        """

        if event.button() == QtCore.Qt.RightButton:
            self._showContextualMenu()
        else:
            super().mousePressEvent(event)

    def _showContextualMenu(self):
        """
        Contextual menu to expand and collapse the tree.
        """

        menu = QtWidgets.QMenu()
        expand_all = QtWidgets.QAction("Expand all", menu)
        expand_all.setIcon(QtGui.QIcon(":/icons/plus.svg"))
        expand_all.triggered.connect(self._expandAllSlot)
        menu.addAction(expand_all)

        collapse_all = QtWidgets.QAction("Collapse all", menu)
        collapse_all.setIcon(QtGui.QIcon(":/icons/minus.svg"))
        collapse_all.triggered.connect(self._collapseAllSlot)
        menu.addAction(collapse_all)

        if self.show_only_devices_with_capture is False and self.show_only_devices_with_filters is False:
            devices_with_capture = QtWidgets.QAction("Show devices with capture(s)", menu)
            devices_with_capture.setIcon(QtGui.QIcon(":/icons/inspect.svg"))
            devices_with_capture.triggered.connect(self._devicesWithCaptureSlot)
            menu.addAction(devices_with_capture)

            devices_with_filters = QtWidgets.QAction("Show devices with packet filter(s)", menu)
            devices_with_filters.setIcon(QtGui.QIcon(":/icons/filter.svg"))
            devices_with_filters.triggered.connect(self._devicesWithFiltersSlot)
            menu.addAction(devices_with_filters)

        else:
            show_all_devices = QtWidgets.QAction("Show all devices", menu)
            # show_all_devices.setIcon(QtGui.QIcon(":/icons/inspect.svg"))
            show_all_devices.triggered.connect(self._showAllDevicesSlot)
            menu.addAction(show_all_devices)

        stop_all_captures = QtWidgets.QAction("Stop all captures", menu)
        stop_all_captures.setIcon(QtGui.QIcon(":/icons/capture-stop.svg"))
        stop_all_captures.triggered.connect(self._stopAllCapturesSlot)
        menu.addAction(stop_all_captures)

        reset_all_filters = QtWidgets.QAction("Reset all packet filters", menu)
        reset_all_filters.setIcon(QtGui.QIcon(":/icons/filter-reset.svg"))
        reset_all_filters.triggered.connect(self._resetAllFiltersSlot)
        menu.addAction(reset_all_filters)

        current_item = self.currentItem()
        from .main_window import MainWindow
        view = MainWindow.instance().uiGraphicsView
        if current_item and not current_item.isHidden():
            menu.addSeparator()
            if isinstance(current_item, TopologyNodeItem):
                view.populateDeviceContextualMenu(menu)
            else:
                link = current_item.data(0, QtCore.Qt.UserRole)
                for item in view.scene().items():
                    if isinstance(item, LinkItem) and item.link() == link:
                        item.populateLinkContextualMenu(menu)
                        break

        menu.exec_(QtGui.QCursor.pos())

    @qslot
    def _expandAllSlot(self, *args):
        """
        Expands all items.
        """

        self.expandAll()

    @qslot
    def _collapseAllSlot(self, *args):
        """
        Collapses all items.
        """

        self.collapseAll()

    @qslot
    def _devicesWithCaptureSlot(self, *args):
        """
        Show only devices with captures.
        """

        self.show_only_devices_with_capture = True
        self.refreshAllLinks()

    @qslot
    def _devicesWithFiltersSlot(self, *args):
        """
        Show only devices with filters.
        """

        self.show_only_devices_with_filters = True
        self.refreshAllLinks()

    @qslot
    def _showAllDevicesSlot(self, *args):
        """
        Show all devices items.
        """

        self.show_only_devices_with_capture = False
        self.show_only_devices_with_filters = False
        self.refreshAllLinks()

    @qslot
    def _stopAllCapturesSlot(self, *args):
        """
        Stop all packet captures.
        """

        for link in self._topology.links():
            if link.capturing():
                PacketCapture.instance().stopCapture(link)

    @qslot
    def _resetAllFiltersSlot(self, *args):
        """
        Reset all packet filters
        """

        for link in self._topology.links():
            if len(link.filters()) > 0:
                filters = {}
                link.setFilters(filters)
                link.update()



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from gns3.compute import Compute


def test_init():
    compute = Compute("local")
    assert compute.id() == "local"


def test_init_without_id():
    compute = Compute()
    assert compute.id() is not None


def test_json():
    compute = Compute("test")
    compute.setHost("example.org")
    compute.setName("Test")
    compute.setProtocol("https")
    compute.setUser("hello")
    compute.setPassword("world")
    compute.setPort(4242)
    assert compute.__json__() == {
        'compute_id': 'test',
        'host': 'example.org',
        'name': 'Test',
        'password': 'world',
        'port': 4242,
        'protocol': 'https',
        'user': 'hello'
    }



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/ui/edit_compute_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.8
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_EditComputeDialog(object):

    def setupUi(self, EditComputeDialog):
        EditComputeDialog.setObjectName("EditComputeDialog")
        EditComputeDialog.setWindowModality(QtCore.Qt.WindowModal)
        EditComputeDialog.resize(579, 374)
        self.verticalLayout = QtWidgets.QVBoxLayout(EditComputeDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox = QtWidgets.QGroupBox(EditComputeDialog)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.uiServerNameLineEdit = QtWidgets.QLineEdit(self.groupBox)
        self.uiServerNameLineEdit.setObjectName("uiServerNameLineEdit")
        self.gridLayout.addWidget(self.uiServerNameLineEdit, 0, 1, 1, 1)
        self.uiServerPortSpinBox = QtWidgets.QSpinBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiServerPortSpinBox.sizePolicy().hasHeightForWidth())
        self.uiServerPortSpinBox.setSizePolicy(sizePolicy)
        self.uiServerPortSpinBox.setSuffix(" TCP")
        self.uiServerPortSpinBox.setMaximum(65535)
        self.uiServerPortSpinBox.setProperty("value", 3080)
        self.uiServerPortSpinBox.setObjectName("uiServerPortSpinBox")
        self.gridLayout.addWidget(self.uiServerPortSpinBox, 2, 1, 1, 2)
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.uiServerHostLineEdit = QtWidgets.QLineEdit(self.groupBox)
        self.uiServerHostLineEdit.setObjectName("uiServerHostLineEdit")
        self.gridLayout.addWidget(self.uiServerHostLineEdit, 1, 1, 1, 2)
        self.uiServerHostLabel = QtWidgets.QLabel(self.groupBox)
        self.uiServerHostLabel.setObjectName("uiServerHostLabel")
        self.gridLayout.addWidget(self.uiServerHostLabel, 1, 0, 1, 1)
        self.uiServerPortLabel = QtWidgets.QLabel(self.groupBox)
        self.uiServerPortLabel.setObjectName("uiServerPortLabel")
        self.gridLayout.addWidget(self.uiServerPortLabel, 2, 0, 1, 1)
        self.verticalLayout.addWidget(self.groupBox)
        self.uiEnableAuthenticationCheckBox = QtWidgets.QGroupBox(EditComputeDialog)
        self.uiEnableAuthenticationCheckBox.setCheckable(True)
        self.uiEnableAuthenticationCheckBox.setObjectName("uiEnableAuthenticationCheckBox")
        self.formLayout = QtWidgets.QFormLayout(self.uiEnableAuthenticationCheckBox)
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.ExpandingFieldsGrow)
        self.formLayout.setObjectName("formLayout")
        self.uiServerUserLabel = QtWidgets.QLabel(self.uiEnableAuthenticationCheckBox)
        self.uiServerUserLabel.setObjectName("uiServerUserLabel")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.uiServerUserLabel)
        self.uiServerUserLineEdit = QtWidgets.QLineEdit(self.uiEnableAuthenticationCheckBox)
        self.uiServerUserLineEdit.setEnabled(True)
        self.uiServerUserLineEdit.setToolTip("")
        self.uiServerUserLineEdit.setObjectName("uiServerUserLineEdit")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.uiServerUserLineEdit)
        self.uiServerPasswordLabel = QtWidgets.QLabel(self.uiEnableAuthenticationCheckBox)
        self.uiServerPasswordLabel.setObjectName("uiServerPasswordLabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.uiServerPasswordLabel)
        self.uiServerPasswordLineEdit = QtWidgets.QLineEdit(self.uiEnableAuthenticationCheckBox)
        self.uiServerPasswordLineEdit.setEnabled(True)
        self.uiServerPasswordLineEdit.setInputMethodHints(QtCore.Qt.ImhHiddenText | QtCore.Qt.ImhNoAutoUppercase | QtCore.Qt.ImhNoPredictiveText | QtCore.Qt.ImhSensitiveData)
        self.uiServerPasswordLineEdit.setEchoMode(QtWidgets.QLineEdit.Password)
        self.uiServerPasswordLineEdit.setObjectName("uiServerPasswordLineEdit")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.uiServerPasswordLineEdit)
        self.verticalLayout.addWidget(self.uiEnableAuthenticationCheckBox)
        self.uiWarningLabel = QtWidgets.QLabel(EditComputeDialog)
        self.uiWarningLabel.setWordWrap(True)
        self.uiWarningLabel.setObjectName("uiWarningLabel")
        self.verticalLayout.addWidget(self.uiWarningLabel)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.buttonBox = QtWidgets.QDialogButtonBox(EditComputeDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel | QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(EditComputeDialog)
        self.buttonBox.accepted.connect(EditComputeDialog.accept)
        self.buttonBox.rejected.connect(EditComputeDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(EditComputeDialog)
        EditComputeDialog.setTabOrder(self.uiServerNameLineEdit, self.uiServerHostLineEdit)
        EditComputeDialog.setTabOrder(self.uiServerHostLineEdit, self.uiServerPortSpinBox)
        EditComputeDialog.setTabOrder(self.uiServerPortSpinBox, self.uiEnableAuthenticationCheckBox)
        EditComputeDialog.setTabOrder(self.uiEnableAuthenticationCheckBox, self.uiServerUserLineEdit)
        EditComputeDialog.setTabOrder(self.uiServerUserLineEdit, self.uiServerPasswordLineEdit)

    def retranslateUi(self, EditComputeDialog):
        _translate = QtCore.QCoreApplication.translate
        EditComputeDialog.setWindowTitle(_translate("EditComputeDialog", "Edit server settings"))
        self.groupBox.setTitle(_translate("EditComputeDialog", "Server settings"))
        self.label.setText(_translate("EditComputeDialog", "Name:"))
        self.uiServerHostLineEdit.setText(_translate("EditComputeDialog", "192.168.56.101"))
        self.uiServerHostLabel.setText(_translate("EditComputeDialog", "Host:"))
        self.uiServerPortLabel.setText(_translate("EditComputeDialog", "Port:"))
        self.uiEnableAuthenticationCheckBox.setTitle(_translate("EditComputeDialog", "Enable authentication"))
        self.uiServerUserLabel.setText(_translate("EditComputeDialog", "User:"))
        self.uiServerPasswordLabel.setText(_translate("EditComputeDialog", "Password:"))
        self.uiWarningLabel.setText(_translate("EditComputeDialog", "<html><head/><body><p><span style=\" font-weight:600;\">WARNING</span>: Changing a server with authentication enabled will reset the password.</p></body></html>"))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Nodes view that list all the available nodes to be dragged and dropped
on the QGraphics scene.
"""

import tempfile
import json
import sip

from .qt import QtCore, QtGui, QtWidgets, qpartial
from .modules import MODULES
from .controller import Controller
from .appliance_manager import ApplianceManager
from .dialogs.configuration_dialog import ConfigurationDialog
from .local_config import LocalConfig


CATEGORY_TO_ID = {
    "firewall": 3,
    "guest": 2,
    "switch": 1,
    "multilayer_switch": 1,
    "router": 0
}


class NodesView(QtWidgets.QTreeWidget):

    """
    Nodes view to list the nodes.

    :param parent: parent widget
    """

    def __init__(self, parent=None):

        super().__init__(parent)
        self._current_category = None
        self._current_search = ""
        self._show_installed_appliances = True
        self._show_builtin_available_appliances = True
        self._show_my_available_appliances = True

        # enables the possibility to drag items.
        self.setDragEnabled(True)

        ApplianceManager.instance().appliances_changed_signal.connect(self.refresh)

    def setCurrentSearch(self, search):
        self._current_search = search

    def setShowInstalledAppliances(self, value):
        self._show_installed_appliances = value

    def setShowBuiltinAvailableAppliances(self, value):
        self._show_builtin_available_appliances = value

    def setShowMyAvailableAppliances(self, value):
        self._show_my_available_appliances = value

    def refresh(self):
        self.clear()
        self.populateNodesView(self._current_category, self._current_search)

    def populateNodesView(self, category, search):
        """
        Populates the nodes view with the device list of the specified
        category (None = all devices).

        :param category: category of device to list
        :param search: filter
        """

        if not Controller.instance().connected():
            return
        self.setIconSize(QtCore.QSize(32, 32))
        self._current_category = category
        self._current_search = search

        display_appliances = set()

        if self._show_installed_appliances:
            for appliance in ApplianceManager.instance().appliances():
                if category is not None and category != CATEGORY_TO_ID[appliance["category"]]:
                    continue
                if search != "" and search.lower() not in appliance["name"].lower():
                    continue

                display_appliances.add(appliance["name"])
                item = QtWidgets.QTreeWidgetItem(self)
                item.setText(0, appliance["name"])
                item.setData(0, QtCore.Qt.UserRole, appliance["appliance_id"])
                item.setData(1, QtCore.Qt.UserRole, "appliance")
                item.setSizeHint(0, QtCore.QSize(32, 32))
                Controller.instance().getSymbolIcon(appliance.get("symbol"), qpartial(self._setItemIcon, item), fallback=":/symbols/" + appliance["category"] + ".svg")

        for appliance in ApplianceManager.instance().appliance_templates():
            if not appliance["builtin"] and not self._show_my_available_appliances:
                continue
            if appliance["builtin"] and not self._show_builtin_available_appliances:
                continue

            if category is not None and category != CATEGORY_TO_ID[appliance["category"]]:
                continue
            if search != "" and search.lower() not in appliance["name"].lower():
                continue
            if appliance["name"] in display_appliances:
                continue

            item = QtWidgets.QTreeWidgetItem(self)
            item.setForeground(0, QtGui.QBrush(QtGui.QColor("gray")))
            item.setText(0, appliance["name"])
            item.setData(0, QtCore.Qt.UserRole, appliance)
            item.setData(1, QtCore.Qt.UserRole, "appliance_template")
            item.setSizeHint(0, QtCore.QSize(32, 32))
            Controller.instance().getSymbolIcon(appliance.get("symbol"), qpartial(self._setItemIcon, item), fallback=":/symbols/" + appliance["category"] + ".svg")
        self.sortByColumn(0, QtCore.Qt.AscendingOrder)

    def _setItemIcon(self, item, icon):
        if not sip.isdeleted(item):
            item.setIcon(0, icon)

    def _getMainWindow(self):
        from .main_window import MainWindow
        window = self.window()
        # when we're docked in main window
        if isinstance(window, MainWindow):
            return window
        # when we're in docked mode, outside main window
        return self.window().parent()

    def mousePressEvent(self, event):
        """
        Handles all mouse press events.

        :param: QMouseEvent instance
        """

        # Check that an item has been selected and right click
        if self.currentItem() is not None and event.button() == QtCore.Qt.RightButton:
            self._showContextualMenu()
            event.accept()
            return
        super().mousePressEvent(event)

    def mouseMoveEvent(self, event):
        """
        Handles all mouse move events.
        This is the starting point to drag & drop a node on the scene.

        :param: QMouseEvent instance
        """

        # Check that an item has been selected and left button clicked
        if self.currentItem() is not None and event.buttons() == QtCore.Qt.LeftButton:
            item = self.currentItem()

            # retrieve the node class from the item data
            if item.data(1, QtCore.Qt.UserRole) == "appliance_template":
                f = tempfile.NamedTemporaryFile(mode="w+", suffix=".builtin.gns3a", delete=False)
                json.dump(item.data(0, QtCore.Qt.UserRole), f)
                f.close()
                self._getMainWindow().loadPath(f.name)
                return

            icon = item.icon(0)
            mimedata = QtCore.QMimeData()

            if item.data(1, QtCore.Qt.UserRole) == "appliance":
                appliance_id = item.data(0, QtCore.Qt.UserRole)
                mimedata.setData("application/x-gns3-appliance", appliance_id.encode())
            elif item.data(1, QtCore.Qt.UserRole) == "node":
                appliance_id = item.data(0, QtCore.Qt.UserRole)
                mimedata.setData("application/x-gns3-appliance", appliance_id.encode())

            drag = QtGui.QDrag(self)
            drag.setMimeData(mimedata)
            drag.setPixmap(icon.pixmap(self.iconSize()))
            drag.setHotSpot(QtCore.QPoint(drag.pixmap().width(), drag.pixmap().height()))
            drag.exec_(QtCore.Qt.CopyAction)
            event.accept()

    def _showContextualMenu(self):
        item = self.currentItem()
        node = ApplianceManager.instance().getAppliance(item.data(0, QtCore.Qt.UserRole))
        if not node:
            return
        for module in MODULES:
            if node["node_type"] == "dynamips":
                node_class = module.getNodeType(node["node_type"], node["platform"])
            else:
                node_class = module.getNodeType(node["node_type"])

            if node_class:
                break

        # We can not edit stuff like EthernetSwitch
        # or without config template like VPCS
        if not node["builtin"] and hasattr(module, "vmConfigurationPage"):
            vm = None
            for vm_key, vm in module.instance().VMs().items():
                if vm["name"] == node["name"]:
                    break
            if vm is None:
                return
            menu = QtWidgets.QMenu()
            configuration = QtWidgets.QAction("Configure Template", menu)
            configuration.setIcon(QtGui.QIcon(":/icons/configuration.svg"))
            configuration.triggered.connect(qpartial(self._configurationSlot, vm, module))
            menu.addAction(configuration)

            configuration = QtWidgets.QAction("Delete Template", menu)
            configuration.setIcon(QtGui.QIcon(":/icons/delete.svg"))
            configuration.triggered.connect(qpartial(self._deleteSlot, vm_key, vm, module))
            menu.addAction(configuration)

            menu.exec_(QtGui.QCursor.pos())

    def _configurationSlot(self, vm, module, source):

        dialog = ConfigurationDialog(vm["name"], vm, module.vmConfigurationPage()(), parent=self)
        dialog.show()
        if dialog.exec_():
            module.instance().setVMs(module.instance().VMs())
            LocalConfig.instance().writeConfig()
            self.refresh()

    def _deleteSlot(self, vm_key, vm, module, source):

        reply = QtWidgets.QMessageBox.question(self, "Template", "Delete {} template?".format(vm["name"]),
                                               QtWidgets.QMessageBox.Yes, QtWidgets.QMessageBox.No)
        if reply == QtWidgets.QMessageBox.Yes:
            vms = module.instance().VMs()
            vms.pop(vm_key)
            module.instance().setVMs(vms)
            LocalConfig.instance().writeConfig()
            self.refresh()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import configparser


import logging
log = logging.getLogger(__name__)


class LocalServerConfig:

    """
    Local server configuration.
    """

    def __init__(self, config_file=None):

        appname = "GNS3"

        self._config = configparser.RawConfigParser()

        if config_file:
            self._config_file = config_file
        else:
            if sys.platform.startswith("win"):
                filename = "gns3_server.ini"
            else:
                filename = "gns3_server.conf"

            from .local_config import LocalConfig
            if sys.platform.startswith("win"):
                self._config_file = os.path.join(LocalConfig.instance().configDirectory(), filename)
            else:
                self._config_file = os.path.join(LocalConfig.instance().configDirectory(), filename)

        try:
            # create the config file if it doesn't exist
            open(self._config_file, "a").close()
        except OSError as e:
            log.error("Could not create the local server configuration {}: {}".format(self._config_file, e))
        self.readConfig()

    def setConfigFile(self, path):
        """
        Change the location of the server config (use for test)
        """
        self._config = configparser.RawConfigParser()
        self._config_file = path
        self.readConfig()

    def readConfig(self):
        """
        Read the configuration file.
        """

        try:
            self._config.read(self._config_file, encoding="utf-8")
        except (OSError, configparser.Error, UnicodeEncodeError, UnicodeDecodeError) as e:
            log.error("Could not read the local server configuration {}: {}".format(self._config_file, e))

    def writeConfig(self):
        """
        Write the configuration file.
        """

        try:
            log.debug("Write configuration file %s", self._config_file)
            with open(self._config_file, "w", encoding="utf-8") as fp:
                self._config.write(fp)
        except (OSError, configparser.Error) as e:
            log.error("Could not write the local server configuration {}: {}".format(self._config_file, e))

    def loadSettings(self, section, default_settings):
        """
        Get all the settings from a given section.

        :param section: section name
        :param default_settings: setting names and default values (dict)

        :returns: settings (dict)
        """

        if section not in self._config:
            self._config[section] = {}

        settings = {}
        for name, default in default_settings.items():
            if isinstance(default, bool):
                settings[name] = self._config[section].getboolean(name, default)
            elif isinstance(default, int):
                settings[name] = self._config[section].getint(name, default)
            elif isinstance(default, float):
                settings[name] = self._config[section].getfloat(name, default)
            else:
                settings[name] = self._config[section].get(name, default)
                if settings[name] == "None":
                    settings[name] = None

        # sync with the config file
        self.saveSettings(section, settings)
        return settings

    def saveSettings(self, section, settings):
        """
        Save all the settings in a given section.

        :param section: section name
        :param settings: settings to save (dict)
        """

        changed = False
        if section not in self._config:
            self._config[section] = {}
            changed = True

        for name, value in settings.items():
            if name not in self._config[section] or self._config[section][name] != str(value):
                self._config[section][name] = str(value)
                changed = True

        if changed:
            self.writeConfig()

    @staticmethod
    def instance():
        """
        Singleton to return only on instance of LocalServerConfig.

        :returns: instance of Config
        """

        if not hasattr(LocalServerConfig, "_instance"):
            LocalServerConfig._instance = LocalServerConfig()
        return LocalServerConfig._instance
